@@ -18,6 +18,13 @@ import os.path
 import unittest
 import virtinst.CapabilitiesParser as capabilities
 
+def build_host_feature_dict(feature_list):
+    fdict = {}
+    for f in feature_list:
+        fdict[f] = capabilities.FEATURE_ON
+
+    return fdict
+
 class TestCapabilities(unittest.TestCase):
 
     def _compareGuest(self, (arch, os_type, domains, features), guest):
@@ -32,9 +39,15 @@ class TestCapabilities(unittest.TestCase):
         for n in features:
             self.assertEqual(features[n],        guest.features[n])
 
+    def _buildCaps(self, filename):
+        path = os.path.join("tests/capabilities-xml", filename)
+        xml = file(path).read()
+
+        return capabilities.parse(xml)
+
     def _testCapabilities(self, path, (host_arch, host_features), guests,
                           secmodel=None):
-        caps = capabilities.parse(file(os.path.join("tests/capabilities-xml", path)).read())
+        caps = self._buildCaps(path)
 
         self.assertEqual(host_arch,     caps.host.arch)
         for n in host_features:
@@ -116,5 +129,63 @@ class TestCapabilities(unittest.TestCase):
 
         self._testCapabilities("capabilities-test.xml", host, guests)
 
+    def testCapsCPUFeaturesOldSyntax(self):
+        filename = "rhel5.4-xen-caps-virt-enabled.xml"
+        host_feature_list = ["vmx"]
+        feature_dict = build_host_feature_dict(host_feature_list)
+
+        caps = self._buildCaps(filename)
+        for f in feature_dict.keys():
+            self.assertEquals(caps.host.features[f], feature_dict[f])
+
+    def testCapsCPUFeaturesOldSyntaxSVM(self):
+        filename = "rhel5.4-xen-caps.xml"
+        host_feature_list = ["svm"]
+        feature_dict = build_host_feature_dict(host_feature_list)
+
+        caps = self._buildCaps(filename)
+        for f in feature_dict.keys():
+            self.assertEquals(caps.host.features[f], feature_dict[f])
+
+    def testCapsCPUFeaturesNewSyntax(self):
+        filename = "libvirt-0.7.6-qemu-caps.xml"
+        host_feature_list = ['lahf_lm', 'xtpr', 'cx16', 'tm2', 'est', 'vmx',
+                             'ds_cpl', 'pbe', 'tm', 'ht', 'ss', 'acpi', 'ds']
+        feature_dict = build_host_feature_dict(host_feature_list)
+
+        caps = self._buildCaps(filename)
+        for f in feature_dict.keys():
+            self.assertEquals(caps.host.features[f], feature_dict[f])
+
+        self.assertEquals(caps.host.cpu.model, "core2duo")
+        self.assertEquals(caps.host.cpu.threads, "3")
+        self.assertEquals(caps.host.cpu.cores, "5")
+        self.assertEquals(caps.host.cpu.sockets, "7")
+
+    def testCapsUtilFuncs(self):
+        new_caps = self._buildCaps("libvirt-0.7.6-qemu-caps.xml")
+        new_caps_no_kvm = self._buildCaps(
+                                    "libvirt-0.7.6-qemu-no-kvmcaps.xml")
+        empty_caps = self._buildCaps("empty-caps.xml")
+        rhel_xen_enable_hvm_caps = self._buildCaps(
+                                    "rhel5.4-xen-caps-virt-enabled.xml")
+        rhel_xen_caps = self._buildCaps("rhel5.4-xen-caps.xml")
+        rhel_kvm_caps = self._buildCaps("rhel5.4-kvm-caps.xml")
+
+        def test_utils(caps, no_guests, is_hvm, is_kvm, is_bios_disable,
+                       is_xenner):
+            self.assertEquals(caps.no_install_options(), no_guests)
+            self.assertEquals(caps.hw_virt_supported(), is_hvm)
+            self.assertEquals(caps.is_kvm_available(), is_kvm)
+            self.assertEquals(caps.is_bios_virt_disabled(), is_bios_disable)
+            self.assertEquals(caps.is_xenner_available(), is_xenner)
+
+        test_utils(new_caps, False, True, True, False, True)
+        test_utils(empty_caps, True, False, False, False, False)
+        test_utils(rhel_xen_enable_hvm_caps, False, True, False, False, False)
+        test_utils(rhel_xen_caps, False, True, False, True, False)
+        test_utils(rhel_kvm_caps, False, True, True, False, False)
+        test_utils(new_caps_no_kvm, False, True, False, False, False)
+
 if __name__ == "__main__":
     unittest.main()
