@@ -19,6 +19,7 @@ import os, sys
 
 # Set DISPLAY if it isn't already set
 os.environ["DISPLAY"] = "testdisplay"
+os.environ["VIRTCONV_TEST_NO_DISK_CONVERSION"] = "1"
 
 testuri = "test:///`pwd`/tests/testdriver.xml"
 
@@ -28,8 +29,12 @@ remoteuri = "__virtinst_test_remote__test:///`pwd`/tests/testdriver.xml"
 
 # Location
 xmldir = "tests/cli-test-xml"
-treedir = "tests/cli-test-xml/faketree"
-ro_img = "cli_exist3ro.img"
+treedir = "%s/faketree" % xmldir
+vcdir = "%s/virtconv" % xmldir
+ro_dir = "clitest_rodir"
+ro_img = "%s/cli_exist3ro.img" % ro_dir
+ro_noexist_img = "%s/idontexist.img" % ro_dir
+virtconv_out = "virtconv-outdir"
 
 # Images that will be created by virt-install/virt-clone, and removed before
 # each run
@@ -45,15 +50,20 @@ virtimage_exist = [os.path.join(xmldir, "cli_root.raw")]
 # Images created by virt-image
 virtimage_new = [os.path.join(xmldir, "cli_scratch.raw")]
 
+# virt-convert output dirs
+virtconv_dirs = [virtconv_out]
+
 exist_files = exist_images + virtimage_exist
-new_files   = new_images + virtimage_new
-clean_files = new_images + exist_images + virtimage_exist + virtimage_new
+new_files   = new_images + virtimage_new + virtconv_dirs
+clean_files = (new_images + exist_images +
+               virtimage_exist + virtimage_new + virtconv_dirs + [ro_dir])
 
 test_files = {
     'TESTURI'           : testuri,
     'REMOTEURI'         : remoteuri,
     'CLONE_DISK_XML'    : "%s/clone-disk.xml" % xmldir,
     'CLONE_STORAGE_XML' : "%s/clone-disk-managed.xml" % xmldir,
+    'CLONE_NOEXIST_XML' : "%s/clone-disk-noexist.xml" % xmldir,
     'IMAGE_XML'         : "%s/image.xml" % xmldir,
     'NEWIMG1'           : new_images[0],
     'NEWIMG2'           : new_images[1],
@@ -61,13 +71,21 @@ test_files = {
     'EXISTIMG1'         : exist_images[0],
     'EXISTIMG2'         : exist_images[1],
     'ROIMG'             : ro_img,
+    'ROIMGNOEXIST'      : ro_noexist_img,
     'POOL'              : "default-pool",
     'VOL'               : "testvol1.img",
+    'DIR'               : os.getcwd(),
     'MANAGEDEXIST1'     : "/default-pool/testvol1.img",
     'MANAGEDEXIST2'     : "/default-pool/testvol2.img",
     'MANAGEDNEW1'       : "/default-pool/clonevol",
     'MANAGEDNEW2'       : "/default-pool/clonevol",
     'MANAGEDDISKNEW1'   : "/disk-pool/newvol1.img",
+    'COLLIDE'           : "/default-pool/collidevol1.img",
+    'SHARE'             : "/default-pool/sharevol.img",
+
+    'VIRTCONV_OUT'      : "%s/test.out" % virtconv_out,
+    'VC_IMG1'           : "%s/virtimage/test1.virt-image" % vcdir,
+    'VMX_IMG1'          : "%s/vmx/test1.vmx" % vcdir,
 }
 
 debug = False
@@ -150,6 +168,14 @@ args_dict = {
         "--disk %(EXISTIMG1)s",
         # Not specifying path= but creating storage
         "--disk %(NEWIMG1)s,format=raw,size=.0000001",
+        # Colliding storage with --force
+        "--disk %(COLLIDE)s --force",
+        # Colliding shareable storage
+        "--disk %(SHARE)s,perms=sh",
+        # Two IDE cds
+        "--disk path=%(EXISTIMG1)s,device=cdrom --disk path=%(EXISTIMG1)s,device=cdrom",
+        # Dir with a floppy dev
+        "--disk %(DIR)s,device=floppy",
       ],
 
       "invalid": [
@@ -161,8 +187,6 @@ args_dict = {
         "--file %(NEWIMG1)s",
         # Too many IDE
         "--file %(EXISTIMG1)s --file %(EXISTIMG1)s --file %(EXISTIMG1)s --file %(EXISTIMG1)s --file %(EXISTIMG1)s",
-        # Two IDE cds
-        "--disk path=%(EXISTIMG1)s,device=cdrom --disk path=%(EXISTIMG1)s,device=cdrom",
         # Size, no file
         "--file-size .0001",
         # Specify a nonexistent pool
@@ -181,6 +205,10 @@ args_dict = {
         "--disk path=%(MANAGEDDISKNEW1)s,format=raw,size=.0000001",
         # Not specifying path= and non existent storage w/ no size
         "--disk %(NEWIMG1)s",
+        # Colliding storage without --force
+        "--disk %(COLLIDE)s",
+        # Dir without floppy
+        "--disk %(DIR)s,device=cdrom",
       ]
      }, # category "storage"
 
@@ -199,9 +227,11 @@ args_dict = {
         # Explicity fullvirt + arch
         "--arch i686 --pxe",
         # Convert i*86 -> i686
-        #"--arch i486 --pxe",
+        "--arch i486 --pxe",
         # Directory tree URL install
         "--hvm --location %s" % treedir,
+        # Directory tree URL install with extra-args
+        "--hvm --location %s --extra-args console=ttyS0" % treedir,
         # Directory tree CDROM install
         "--hvm --cdrom %s" % treedir,
         # Paravirt location
@@ -324,11 +354,37 @@ args_dict = {
         "--hvm --import --disk path=virt-install",
         # PV Import install
         "--paravirt --import --disk path=virt-install",
+        # --autostart flag
+        "--hvm --nodisks --pxe --autostart",
+        # --description
+        "--hvm --nodisks --pxe --description \"foobar & baz\"",
+        # HVM windows install with disk
+        "--hvm --cdrom %(EXISTIMG2)s --file %(EXISTIMG1)s --os-variant win2k3 --wait 0",
+        # --watchdog dev default
+        "--hvm --nodisks --pxe --watchdog default",
+        # --watchdog opts
+        "--hvm --nodisks --pxe --watchdog ib700,action=pause",
+        # --sound option
+        "--hvm --nodisks --pxe --sound",
+        # --soundhw option
+        "--hvm --nodisks --pxe --soundhw default --soundhw ac97",
+        # --security dynamic
+        "--hvm --nodisks --pxe --security type=dynamic",
+        # --security implicit static
+        "--hvm --nodisks --pxe --security label=foobar.label",
       ],
 
       "invalid": [
         # pxe and nonetworks
         "--nodisks --pxe --nonetworks",
+        # Colliding name
+        "--nodisks --pxe --name test",
+        # Busted --watchdog
+        "--hvm --nodisks --pxe --watchdog default,action=foobar",
+        # Busted --soundhw
+        "--hvm --nodisks --pxe --soundhw default --soundhw foobar",
+        # Busted --security
+        "--hvm --nodisks --pxe --security type=foobar",
       ],
      }, # category "misc"
 
@@ -337,13 +393,13 @@ args_dict = {
 
       "valid": [
         # Just a macaddr
-        "--mac 11:22:33:44:55:66",
+        "--mac 11:22:33:44:55:AF",
         # user networking
         "--network=user",
         # Old bridge option
         "--bridge mybr0",
         # Old bridge w/ mac
-        "--bridge mybr0 --mac 11:22:33:44:55:66",
+        "--bridge mybr0 --mac 11:22:33:44:55:AF",
         # --network bridge:
         "--network bridge:mybr0,model=e1000",
         # VirtualNetwork with a random macaddr
@@ -355,7 +411,7 @@ args_dict = {
         # with NIC model
         "--network=user,model=e1000",
         # several networks
-        "--network=network:default,model=e1000 --network=user,model=virtio,mac=11:22:33:44:55:66",
+        "--network=network:default,model=e1000 --network=user,model=virtio,mac=11:22:33:44:55:AF",
       ],
       "invalid": [
         # Nonexistent network
@@ -363,13 +419,35 @@ args_dict = {
         # Invalid mac
         "--network=network:default --mac 1234",
         # More mac addrs than nics
-        "--network user --mac 00:11:22:33:44:55 --mac 00:11:22:33:44:66",
+        "--network user --mac 00:11:22:33:44:EF --mac 00:11:22:33:44:AB",
         # Mixing bridge and network
         "--network user --bridge foo0",
+        # Colliding macaddr
+        "--mac 11:22:33:44:55:66",
       ],
 
      }, # category "network"
 
+     "hostdev" : {
+      "hostdev_args": "--noautoconsole --nographics --nodisks --pxe",
+
+      "valid" : [
+        # Host dev by libvirt name
+        "--host-device usb_device_781_5151_2004453082054CA1BEEE",
+        # Many hostdev parsing types
+        "--host-device 001.003 --host-device 15:0.1 --host-device 2:15:0.2 --host-device 0:15:0.3 --host-device 0x0781:0x5151 --host-device 1d6b:2",
+      ],
+
+      "invalid" : [
+        # Unsupported hostdev type
+        "--host-device pci_8086_2850_scsi_host_scsi_host",
+        # Unknown hostdev
+        "--host-device foobarhostdev",
+        # Parseable hostdev, but unknown digits
+        "--host-device 300:400",
+      ],
+     }, # category "hostdev"
+
      "remote" : {
       "remote_args": "--connect %(REMOTEURI)s --nographics --noautoconsole",
 
@@ -423,7 +501,9 @@ args_dict = {
         # XML w/ managed storage, specify managed path
         "--original-xml %(CLONE_STORAGE_XML)s --file %(MANAGEDNEW1)s",
         # XML w/ managed storage, specify managed path across pools
-        #"--original-xml %(CLONE_STORAGE_XML)s --file /cross-pool/clonevol",
+        "--original-xml %(CLONE_STORAGE_XML)s --file /cross-pool/clonevol",
+        # XML w/ non-existent storage, with --preserve
+        "--original-xml %(CLONE_NOEXIST_XML)s --file %(EXISTIMG1)s --preserve",
       ],
 
       "invalid": [
@@ -441,6 +521,12 @@ args_dict = {
         "--original-xml %(CLONE_DISK_XML)s --file %(NEWIMG1)s --file %(NEWIMG2)s --force-copy=hdc",
         # XML w/ managed storage, specify unmanaged path (should fail)
         "--original-xml %(CLONE_STORAGE_XML)s --file /tmp/clonevol",
+        # XML w/ non-existent storage, WITHOUT --preserve
+        "--original-xml %(CLONE_NOEXIST_XML)s --file %(EXISTIMG1)s",
+        # XML w/ managed storage, specify RO image without preserve
+        "--original-xml %(CLONE_DISK_XML)s --file %(ROIMG)s --file %(ROIMG)s --force",
+        # XML w/ managed storage, specify RO non existent
+        "--original-xml %(CLONE_DISK_XML)s --file %(ROIMG)s --file %(ROIMGNOEXIST)s --force",
       ]
      }, # category "general"
 
@@ -525,10 +611,15 @@ args_dict = {
     "misc": {
      "misc_args" : "",
 
-      "valid" : [],
+      "valid" : [
+        # Colliding VM name w/ --replace
+        "--name test --replace",
+      ],
       "invalid" : [
         # No name specified, and no prompt flag
         "",
+        # Colliding VM name without --replace
+        "--name test",
       ],
 
      }, # category 'misc'
@@ -558,9 +649,44 @@ args_dict = {
       ],
 
      }, # category "network"
-    "prompt" : [ " --connect %(TESTURI) %(IMAGE_XML)s --debug --prompt" ],
+    "prompt" : [ " --connect %(TESTURI)s %(IMAGE_XML)s --debug --prompt" ],
 
-  } # app 'virt-image'
+  }, # app 'virt-image'
+
+
+  "virt-convert" : {
+    "global_args" : "--debug",
+
+    "misc" : {
+     "misc_args": "",
+
+     "valid": [
+        # virt-image to default (virt-image) w/ no convert
+        "%(VC_IMG1)s -D none %(VIRTCONV_OUT)s",
+        # virt-image to virt-image w/ no convert
+        "%(VC_IMG1)s -o virt-image -D none %(VIRTCONV_OUT)s",
+        # virt-image to vmx w/ no convert
+        "%(VC_IMG1)s -o vmx -D none %(VIRTCONV_OUT)s",
+        # virt-image to vmx w/ raw
+        "%(VC_IMG1)s -o vmx -D raw %(VIRTCONV_OUT)s",
+        # virt-image to vmx w/ vmdk
+        "%(VC_IMG1)s -o vmx -D vmdk %(VIRTCONV_OUT)s",
+        # virt-image to vmx w/ qcow2
+        "%(VC_IMG1)s -o vmx -D qcow2 %(VIRTCONV_OUT)s",
+        # vmx to vmx no convert
+        "%(VMX_IMG1)s -o vmx -D none %(VIRTCONV_OUT)s",
+     ],
+
+     "invalid": [
+        # virt-image to virt-image with invalid format
+        "%(VC_IMG1)s -o virt-image -D foobarfmt %(VIRTCONV_OUT)s",
+        # virt-image to ovf (has no output formatter)
+        "%(VC_IMG1)s -o ovf %(VIRTCONV_OUT)s",
+     ]
+
+    }, # category 'misc'
+
+  }, # app 'virt-conver'
 }
 
 def runcomm(comm):
@@ -601,8 +727,14 @@ def assertFail(comm):
                              "Output was:\n%s" % ret[1])
 
 # Setup: build cliarg dict, which uses
-def run_tests():
+def run_tests(do_app):
+    if do_app and do_app not in args_dict.keys():
+        raise ValueError("Unknown app '%s'" % do_app)
+
     for app in args_dict:
+        if do_app and app != do_app:
+            continue
+
         unique = {}
         prompts = []
         global_args = ""
@@ -646,30 +778,42 @@ def main():
     global debug
     global testprompt
 
+    do_app = None
+
     if len(sys.argv) > 1:
         for i in range(1, len(sys.argv)):
             if sys.argv[i].count("debug"):
                 debug = True
             elif sys.argv[i].count("prompt"):
                 testprompt = True
+            elif sys.argv[i].count("--app"):
+                do_app = sys.argv[i+1]
 
     # Setup needed files
     for i in exist_files:
         if os.path.exists(i):
             raise ValueError("'%s' will be used by testsuite, can not already"
                              " exist." % i)
+
+    os.system("mkdir %s" % ro_dir)
+
     for i in exist_files:
         os.system("touch %s" % i)
 
     # Set ro_img to readonly
     os.system("chmod 444 %s" % ro_img)
+    os.system("chmod 555 %s" % ro_dir)
 
     try:
-        run_tests()
+        run_tests(do_app)
     finally:
-        # Cleanup files
-        for i in clean_files:
-            os.system("rm -f %s > /dev/null 2>&1" % i)
+        cleanup()
+
+def cleanup():
+    # Cleanup files
+    for i in clean_files:
+        os.system("chmod 777 %s > /dev/null 2>&1" % i)
+        os.system("rm -rf %s > /dev/null 2>&1" % i)
 
 if __name__ == "__main__":
     main()
