@@ -37,19 +37,18 @@ POOL1 = "/default-pool"
 POOL2 = "/cross-pool"
 DISKPOOL = "/disk-pool"
 
-for f in [ FILE1, FILE2 ]:
-    os.system("touch %s" % f)
+local_files = [ FILE1, FILE2]
 
 clonexml_dir = os.path.join(os.getcwd(), "tests/clone-xml")
 clone_files = []
 
-for f in os.listdir(clonexml_dir):
+for tmpf in os.listdir(clonexml_dir):
     black_list = [ "managed-storage", "cross-pool", "force", "skip",
                    "fullpool"]
-    if f.endswith("-out.xml"):
-        f = f[0:(len(f) - len("-out.xml"))]
-        if f not in clone_files and f not in black_list:
-            clone_files.append(f)
+    if tmpf.endswith("-out.xml"):
+        tmpf = tmpf[0:(len(tmpf) - len("-out.xml"))]
+        if tmpf not in clone_files and tmpf not in black_list:
+            clone_files.append(tmpf)
 
 conn = tests.open_testdriver()
 
@@ -59,7 +58,12 @@ def fake_is_uri_remote(ignore):
 class TestClone(unittest.TestCase):
 
     def setUp(self):
-        pass
+        for f in local_files:
+            os.system("touch %s" % f)
+
+    def tearDown(self):
+        for f in local_files:
+            os.unlink(f)
 
     def _clone_helper(self, filebase, disks=None, force_list=None,
                       skip_list=None, compare=True):
