@@ -44,6 +44,7 @@ LOCAL_MEDIA = []
 FEDORA_BASEURL = "http://download.fedoraproject.org/pub/fedora/linux/releases/%s/Fedora/%s/os/"
 FEDORA_RAWHIDE_BASEURL = "http://download.fedoraproject.org/pub/fedora/linux/development/%s/os"
 OPENSUSE_BASEURL = "http://download.opensuse.org/distribution/%s/repo/oss/"
+OLD_OPENSUSE_BASEURL = "http://ftp5.gwdg.de/pub/opensuse/discontinued/distribution/%s/repo/oss"
 
 # ISO Code specific URLs
 UBUNTU_BASEURL="http://us.archive.ubuntu.com/ubuntu/dists/%s/main/installer-%s"
@@ -59,12 +60,12 @@ SCIENTIFIC_BASEURL = "http://ftp.scientificlinux.org/linux/scientific/%s/%s/"
 NOXEN_FILTER=".*ubuntu.*|.*etch.*|.*mandriva.*|.*lenny-64.*|.*centos-4.0.*|.*scientific-4.0.*"
 
 # Doesn't appear to be a simple boot iso in newer suse trees
-NOBOOTISO_FILTER=".*opensuse11.*|.*opensuse10.3.*"
+NOBOOTISO_FILTER=".*opensuse11.*|.*opensuse10.3.*|.*opensuse10.0.*"
 
 # Opensuse < 10.3 (and some sles) require crazy rpm hacking to get a bootable
 # kernel. We expect failure in this case since our test harness doesn't
 # actually fetch anything
-EXPECT_XEN_FAIL=".*opensuse10.2.*"
+EXPECT_XEN_FAIL=".*opensuse10.2.*|.*opensuse10.0.*"
 
 # Return the expected Distro class for the passed distro label
 def distroClass(distname):
@@ -125,13 +126,17 @@ urls = {
     },
 
     # SUSE Distros
+    "opensuse10.0" : {
+        'i386'  : "http://ftp.hosteurope.de/mirror/ftp.opensuse.org/discontinued/10.0/",
+        'x86_64': "http://ftp.hosteurope.de/mirror/ftp.opensuse.org/discontinued/10.0/",
+    },
     "opensuse10.2" : {
-        'i386'  : OPENSUSE_BASEURL % ("10.2"),
-        'x86_64': OPENSUSE_BASEURL % ("10.2")
+        'i386'  : OLD_OPENSUSE_BASEURL % ("10.2"),
+        'x86_64': OLD_OPENSUSE_BASEURL % ("10.2")
     },
     "opensuse10.3" : {
-        'i386'  : OPENSUSE_BASEURL % ("10.3"),
-        'x86_64': OPENSUSE_BASEURL % ("10.3")
+        'i386'  : OLD_OPENSUSE_BASEURL % ("10.3"),
+        'x86_64': OLD_OPENSUSE_BASEURL % ("10.3")
     },
     "opensuse11" : {
         'i386'  : OPENSUSE_BASEURL % ("11.0"),
@@ -160,7 +165,7 @@ urls = {
     # CentOS Distros
     "centos-5-latest" : {
         'i386' : CURCENTOS_BASEURL % ("5", "i386"),
-        #'x86_64' : CURCENTOS_BASEURL % ("5", "x86_64"),  # No .treeinfo
+        'x86_64' : CURCENTOS_BASEURL % ("5", "x86_64"),  # No .treeinfo
         'distro': ("linux", "rhel5.4")
     },
     "centos-4-latest" : {
@@ -244,7 +249,7 @@ urls = {
 
 
 testconn = libvirt.open("test:///default")
-testguest = virtinst.Guest(connection=testconn, installer=virtinst.DistroInstaller())
+testguest = virtinst.FullVirtGuest(connection=testconn, installer=virtinst.DistroInstaller())
 
 class TestURLFetch(unittest.TestCase):
 
@@ -362,8 +367,8 @@ class TestURLFetch(unittest.TestCase):
                 if boot != True:
                     raise RuntimeError("Didn't fetch any boot iso.")
         except Exception, e:
-            logging.error("%s-%s: bootdisk fetching: %s" % (distname, arch,
-                                                            str(e)))
+            logging.exception("%s-%s: bootdisk fetching: %s" %
+                              (distname, arch, str(e)))
             self.fail()
 
         # Fetch regular kernel
@@ -374,8 +379,8 @@ class TestURLFetch(unittest.TestCase):
             if kern[0] is not True or kern[1] is not True:
                 raise RuntimeError("Didn't fetch any hvm kernel.")
         except Exception, e:
-            logging.error("%s-%s: hvm kernel fetching: %s" % (distname, arch,
-                                                              str(e)))
+            logging.exception("%s-%s: hvm kernel fetching: %s" %
+                              (distname, arch, str(e)))
             self.fail()
 
         # Fetch xen kernel
@@ -392,9 +397,8 @@ class TestURLFetch(unittest.TestCase):
             if re.match(r"%s" % EXPECT_XEN_FAIL, distname):
                 logging.debug("%s: anticipated xen failure." % distname)
             else:
-                logging.error("%s-%s: xen kernel fetching: %s" % (distname,
-                                                                  arch,
-                                                                  str(e)))
+                logging.exception("%s-%s: xen kernel fetching: %s" %
+                                  (distname, arch, str(e)))
                 self.fail()
 
     def _getStore(self, fetcher, url, _type, arch):
