@@ -26,6 +26,8 @@ from virtinst import VirtualNetworkInterface
 from virtinst import VirtualHostDeviceUSB, VirtualHostDevicePCI
 from virtinst import VirtualCharDevice
 from virtinst import VirtualVideoDevice
+from virtinst import VirtualController
+from virtinst import VirtualWatchdog
 import tests
 
 conn = tests.open_testdriver()
@@ -73,13 +75,20 @@ def get_virtual_network():
     dev.network = "default"
     return dev
 
+def qemu_uri():
+    return "qemu:///system"
+
+def xen_uri():
+    return "xen:///"
+
 class TestXMLConfig(unittest.TestCase):
 
-    def _compare(self, xenguest, filebase, install):
+    def _compare(self, xenguest, filebase, do_install, do_disk_boot=False):
         filename = os.path.join("tests/xmlconfig-xml", filebase + ".xml")
         xenguest._prepare_install(progress.BaseMeter())
         try:
-            actualXML = xenguest.get_config_xml(install=install)
+            actualXML = xenguest.get_config_xml(install=do_install,
+                                                disk_boot=do_disk_boot)
             tests.diff_compare(actualXML, filename)
             # Libvirt throws errors since we are defining domain
             # type='xen', when test driver can only handle type='test'
@@ -94,8 +103,45 @@ class TestXMLConfig(unittest.TestCase):
             dom.destroy()
             dom.undefine()
         finally:
-            xenguest.installer.cleanup()
+            xenguest._cleanup_install()
 
+    def conn_function_wrappers(self, guest, filename, do_boot,
+                               conn_version=None,
+                               conn_uri=None,
+                               libvirt_version=None,
+                               do_disk_boot=False):
+        testconn = guest.conn
+
+        def set_func(newfunc, funcname, obj, force=False):
+            if newfunc or force:
+                orig = None
+                if hasattr(obj, funcname):
+                    orig = getattr(obj, funcname)
+
+                setattr(obj, funcname, newfunc)
+                return orig, True
+
+            return None, False
+
+        def set_version(newfunc, force=False):
+            return set_func(newfunc, "getVersion", testconn, force)
+        def set_uri(newfunc, force=False):
+            return set_func(newfunc, "getURI", testconn, force)
+        def set_libvirt_version(newfunc, force=False):
+            return set_func(newfunc, "getVersion", libvirt, force)
+
+        old_version = None
+        old_uri = None
+        old_libvirt_version = None
+        try:
+            old_version = set_version(conn_version)
+            old_uri = set_uri(conn_uri)
+            old_libvirt_version = set_libvirt_version(libvirt_version)
+            self._compare(guest, filename, do_boot, do_disk_boot)
+        finally:
+            set_version(*old_version)
+            set_uri(*old_uri)
+            set_libvirt_version(*old_libvirt_version)
 
     def testBootParavirtDiskFile(self):
         g = get_basic_paravirt_guest()
@@ -289,30 +335,20 @@ class TestXMLConfig(unittest.TestCase):
         self._compare(g, "install-paravirt-import", False)
 
     def testQEMUDriverName(self):
-        # Swap out _util.get_uri_driver for a fake method that always
-        # returns a qemu driver, to trick VirtualDisk into giving us what we
-        # want
-        def new_get_uri(ignore):
-            return "qemu:///system"
+        g = get_basic_fullyvirt_guest()
+        g.disks.append(get_blkdisk())
+        self.conn_function_wrappers(g, "misc-qemu-driver-name", True,
+                                    conn_uri=qemu_uri)
 
-        oldgetdriver = VirtualDisk._get_uri
-        try:
-            VirtualDisk._get_uri = new_get_uri
-            g = get_basic_fullyvirt_guest()
-            g.disks.append(get_blkdisk())
-            self._compare(g, "misc-qemu-driver-name", True)
+        g = get_basic_fullyvirt_guest()
+        g.disks.append(get_filedisk())
+        self.conn_function_wrappers(g, "misc-qemu-driver-type", True,
+                                    conn_uri=qemu_uri)
 
-            VirtualDisk._get_uri = new_get_uri
-            g = get_basic_fullyvirt_guest()
-            g.disks.append(get_filedisk())
-            self._compare(g, "misc-qemu-driver-type", True)
-
-            VirtualDisk._get_uri = new_get_uri
-            g = get_basic_fullyvirt_guest()
-            g.disks.append(get_filedisk("/default-pool/iso-vol"))
-            self._compare(g, "misc-qemu-iso-disk", True)
-        finally:
-            VirtualDisk._get_uri = oldgetdriver
+        g = get_basic_fullyvirt_guest()
+        g.disks.append(get_filedisk("/default-pool/iso-vol"))
+        self.conn_function_wrappers(g, "misc-qemu-iso-disk", True,
+                                    conn_uri=qemu_uri)
 
     def testXMLEscaping(self):
         g = get_basic_fullyvirt_guest()
@@ -329,7 +365,8 @@ class TestXMLConfig(unittest.TestCase):
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
-        self._compare(g, "install-f10", True)
+        self.conn_function_wrappers(g, "install-f10", True,
+                                    conn_uri=qemu_uri)
 
     def testF11(self):
         g = get_basic_fullyvirt_guest("kvm")
@@ -343,7 +380,66 @@ class TestXMLConfig(unittest.TestCase):
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
-        self._compare(g, "install-f11", False)
+        self.conn_function_wrappers(g, "install-f11", False,
+                                    conn_uri=qemu_uri)
+
+    def testF11AC97(self):
+        def build_guest():
+            g = get_basic_fullyvirt_guest("kvm")
+            g.os_type = "linux"
+            g.os_variant = "fedora11"
+            g.installer = virtinst.DistroInstaller(type="kvm", os_type="hvm",
+                            conn=g.conn, location="/default-pool/default-vol")
+            g.installer.cdrom = True
+            g.disks.append(get_floppy())
+            g.disks.append(get_filedisk())
+            g.disks.append(get_blkdisk())
+            g.nics.append(get_virtual_network())
+            g.add_device(VirtualAudio())
+            return g
+
+        def libvirt_nosupport_ac97(drv=None):
+            libver = 5000
+            if drv:
+                return (libver, libver)
+            return libver
+
+        def conn_nosupport_ac97():
+            return 10000
+
+        def conn_support_ac97():
+            return 11000
+
+        g = build_guest()
+        self.conn_function_wrappers(g, "install-f11-ac97", False,
+                                    conn_uri=qemu_uri,
+                                    conn_version=conn_support_ac97)
+
+        g = build_guest()
+        self.conn_function_wrappers(g, "install-f11-noac97", False,
+                                    libvirt_version=libvirt_nosupport_ac97,
+                                    conn_uri=qemu_uri)
+
+        g = build_guest()
+        self.conn_function_wrappers(g, "install-f11-noac97", False,
+                                    conn_version=conn_nosupport_ac97,
+                                    conn_uri=qemu_uri)
+
+
+    def testF11Qemu(self):
+        g = get_basic_fullyvirt_guest("qemu")
+        g.os_type = "linux"
+        g.os_variant = "fedora11"
+        g.installer = virtinst.DistroInstaller(type="qemu", os_type="hvm",
+                                               conn=g.conn,
+                                               location="/default-pool/default-vol")
+        g.installer.cdrom = True
+        g.disks.append(get_floppy())
+        g.disks.append(get_filedisk())
+        g.disks.append(get_blkdisk())
+        g.nics.append(get_virtual_network())
+        self.conn_function_wrappers(g, "install-f11-qemu", False,
+                                    conn_uri=qemu_uri)
 
     def testF11Xen(self):
         g = get_basic_fullyvirt_guest("xen")
@@ -357,39 +453,45 @@ class TestXMLConfig(unittest.TestCase):
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
-        self._compare(g, "install-f11-xen", False)
+        self.conn_function_wrappers(g, "install-f11-xen", False,
+                                    conn_uri=xen_uri)
 
-    def testBootWindows(self):
+    def _build_win_kvm(self):
         g = get_basic_fullyvirt_guest("kvm")
         g.os_type = "windows"
         g.os_variant = "winxp"
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
-        self._compare(g, "boot-windowsxp-kvm", False)
+        g.add_device(VirtualAudio())
+        g.add_device(VirtualVideoDevice(g.conn))
+
+        return g
 
     def testInstallWindowsKVM(self):
-        g = get_basic_fullyvirt_guest("kvm")
-        g.os_type = "windows"
-        g.os_variant = "winxp"
-        g.disks.append(get_filedisk())
-        g.disks.append(get_blkdisk())
-        g.nics.append(get_virtual_network())
-        self._compare(g, "install-windowsxp-kvm", True)
+        g = self._build_win_kvm()
+        self.conn_function_wrappers(g, "winxp-kvm-stage1", True,
+                                    conn_uri=qemu_uri)
+
+    def testContinueWindowsKVM(self):
+        g = self._build_win_kvm()
+        self.conn_function_wrappers(g, "winxp-kvm-stage2", True,
+                                    conn_uri=qemu_uri,
+                                    do_disk_boot=True)
+
+    def testBootWindowsKVM(self):
+        g = self._build_win_kvm()
+        self.conn_function_wrappers(g, "winxp-kvm-stage3", False,
+                                    conn_uri=qemu_uri)
+
 
     def testInstallWindowsXenNew(self):
-        orig_ver_func = libvirt.getVersion
-        def old_xen_ver(drv=None):
-            libver = orig_ver_func()
-            if drv:
-                return (libver, 3000001)
-            return orig_ver_func(drv)
+        def old_xen_ver():
+            return 3000001
+
+        def new_xen_ver():
+            return 3100000
 
-        def new_xen_ver(drv=None):
-            libver = orig_ver_func()
-            if drv:
-                return (libver, 3100000)
-            return orig_ver_func(drv)
 
         g = get_basic_fullyvirt_guest("xen")
         g.os_type = "windows"
@@ -397,14 +499,14 @@ class TestXMLConfig(unittest.TestCase):
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
+        g.add_device(VirtualAudio())
 
-        try:
-            for f, xml in [(old_xen_ver, "install-windowsxp-xenold"),
-                           (new_xen_ver, "install-windowsxp-xennew")]:
-                libvirt.getVersion = f
-                self._compare(g, xml, True)
-        finally:
-            libvirt.getVersion = orig_ver_func
+        for f, xml in [(old_xen_ver, "install-windowsxp-xenold"),
+                       (new_xen_ver, "install-windowsxp-xennew")]:
+
+            self.conn_function_wrappers(g, xml, True,
+                                        conn_version=f,
+                                        conn_uri=xen_uri)
 
 
     # Device heavy configurations
@@ -468,6 +570,7 @@ class TestXMLConfig(unittest.TestCase):
         g.sound_devs.append(VirtualAudio("sb16", conn=g.conn))
         g.sound_devs.append(VirtualAudio("es1370", conn=g.conn))
         g.sound_devs.append(VirtualAudio("pcspk", conn=g.conn))
+        g.sound_devs.append(VirtualAudio(conn=g.conn))
 
         g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
                                             conn=g.conn)
@@ -506,6 +609,7 @@ class TestXMLConfig(unittest.TestCase):
 
     def testManyDevices(self):
         g = get_basic_fullyvirt_guest()
+        g.description = "foooo barrrr \n baz && snarf. '' \"\" @@$\n"
 
         # Hostdevs
         dev1 = VirtualHostDeviceUSB(g.conn)
@@ -522,10 +626,20 @@ class TestXMLConfig(unittest.TestCase):
                                    device=VirtualDisk.DEVICE_FLOPPY))
         g.disks.append(VirtualDisk(conn=g.conn, path="/dev/loop0",
                                    bus="scsi"))
+        g.disks.append(VirtualDisk(conn=g.conn, path="/tmp", device="floppy"))
         d3 = VirtualDisk(conn=g.conn, path="/default-pool/testvol1.img",
                          bus="scsi", driverName="qemu")
         g.disks.append(d3)
 
+        # Controller devices
+        c1 = VirtualController.get_class_for_type(VirtualController.CONTROLLER_TYPE_IDE)(g.conn)
+        c1.index = "3"
+        c2 = VirtualController.get_class_for_type(VirtualController.CONTROLLER_TYPE_VIRTIOSERIAL)(g.conn)
+        c2.ports = "32"
+        c2.vectors = "17"
+        g.add_device(c1)
+        g.add_device(c2)
+
         # Network devices
         net1 = get_virtual_network()
         net1.model = "e1000"
@@ -553,8 +667,25 @@ class TestXMLConfig(unittest.TestCase):
         vdev2.model_type = "cirrus"
         vdev2.vram = 10 * 1024
         vdev2.heads = 3
+
+        vdev3 = VirtualVideoDevice(g.conn)
         g.add_device(vdev1)
         g.add_device(vdev2)
+        g.add_device(vdev3)
+
+        wdev2 = VirtualWatchdog(g.conn)
+        wdev2.model = "ib700"
+        wdev2.action = "none"
+        g.add_device(wdev2)
+
+        g.clock.offset = "localtime"
+
+        seclabel = virtinst.Seclabel(g.conn)
+        seclabel.type = seclabel.SECLABEL_TYPE_STATIC
+        seclabel.model = "selinux"
+        seclabel.label = "foolabel"
+        seclabel.imagelabel = "imagelabel"
+        g.seclabel = seclabel
 
         g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
                                             conn=g.conn)
