@@ -51,8 +51,19 @@ class Features(object):
 
     def parseXML(self, node):
         d = self.features
-        for n in node.xpathEval("*"):
-            feature = n.name
+
+        feature_list = []
+        if node.name == "features":
+            node_list = node.xpathEval("*")
+            for n in node_list:
+                feature_list.append(n.name)
+        else:
+            # New style features
+            node_list = node.xpathEval("feature/@name")
+            for n in node_list:
+                feature_list.append(n.content)
+
+        for feature in feature_list:
             if not d.has_key(feature):
                 d[feature] = 0
 
@@ -72,6 +83,7 @@ class CapabilityFeatures(Features):
         toggle = xpathString(n, "@toggle")
 
         if default is not None:
+            # Format for guest features
             if default == "on":
                 d[feature] = FEATURE_ON
             elif default == "off":
@@ -81,23 +93,78 @@ class CapabilityFeatures(Features):
             if toggle == "yes":
                 d[feature] |= d[feature] ^ (FEATURE_ON|FEATURE_OFF)
         else:
+            # Format for old HOST features, on OLD old guest features
+            # back compat is just <$featurename>, like <svm/>
             if feature == "nonpae":
                 d["pae"] |= FEATURE_OFF
             else:
                 d[feature] |= FEATURE_ON
 
-class Host(object):
-    def __init__(self, node = None):
+class CPU(object):
+    def __init__(self, node=None):
         # e.g. "i686" or "x86_64"
         self.arch = None
-
+        self.model = None
+        self.sockets = 1
+        self.cores = 1
+        self.threads = 1
         self.features = CapabilityFeatures()
+
+        if not node is None:
+            self.parseXML(node)
+
+    def parseXML(self, node):
+        newstyle_features = False
+
+        child = node.children
+        while child:
+            # Do a first pass to try and detect new style features
+            if child.name == "feature":
+                newstyle_features = True
+                break
+            child = child.next
+
+        if newstyle_features:
+            self.features = CapabilityFeatures(node)
+
+        child = node.children
+        while child:
+            if child.name == "arch":
+                self.arch = child.content
+            elif child.name == "model":
+                self.model = child.content
+            elif child.name == "topology":
+                self.sockets = xpathString(child, "@sockets") or 1
+                self.cores = xpathString(child, "@cores") or 1
+                self.threads = xpathString(child, "@threads") or 1
+
+            elif child.name == "features" and not newstyle_features:
+                self.features = CapabilityFeatures(child)
+
+            child = child.next
+
+class Host(object):
+    def __init__(self, node = None):
+        self.cpu = CPU()
         self.topology = None
         self.secmodel = None
 
         if not node is None:
             self.parseXML(node)
 
+    # Back compat for CPU class
+    def get_arch(self):
+        return self.cpu.arch
+    def set_arch(self, val):
+        self.cpu.arch = val
+    arch = property(get_arch, set_arch)
+
+    def get_features(self):
+        return self.cpu.features
+    def set_features(self, val):
+        self.cpu.features = val
+    features = property(get_features, set_features)
+
     def parseXML(self, node):
         child = node.children
         while child:
@@ -107,17 +174,8 @@ class Host(object):
             if child.name == "secmodel":
                 self.secmodel = SecurityModel(child)
 
-            if child.name != "cpu":
-                child = child.next
-                continue
-
-            n = child.children
-            while n:
-                if n.name == "arch":
-                    self.arch = n.content
-                elif n.name == "features":
-                    self.features = CapabilityFeatures(n)
-                n = n.next
+            if child.name == "cpu":
+                self.cpu = CPU(child)
 
             child = child.next
 
@@ -283,6 +341,107 @@ class Capabilities(object):
 
         self._fixBrokenEmulator()
 
+    def _is_xen(self):
+        for g in self.guests:
+            if g.os_type != "xen":
+                continue
+
+            for d in g.domains:
+                if d.hypervisor_type == "xen":
+                    return True
+
+        return False
+
+    def no_install_options(self):
+        """
+        Return True if there are no install options available
+        """
+        for g in self.guests:
+            if len(g.domains) > 0:
+                return False
+
+        return True
+
+    def hw_virt_supported(self):
+        """
+        Return True if the machine supports hardware virtualization.
+
+        For some cases (like qemu caps pre libvirt 0.7.4) this info isn't
+        sufficiently provided, so we will return True in cases that we
+        aren't sure.
+        """
+        # Obvious case of feature being specified
+        if (self.host.features["vmx"] == FEATURE_ON or
+            self.host.features["svm"] == FEATURE_ON):
+            return True
+
+        # If there is other features, but no virt bit, then HW virt
+        # isn't supported
+        if len(self.host.features.names()):
+            return False
+
+        # Xen caps have always shown this info, so if we didn't find any
+        # features, the host really doesn't have the necc support
+        if self._is_xen():
+            return False
+
+        # Otherwise, we can't be sure, because there was a period for along
+        # time that qemu caps gave no indication one way or the other.
+        return True
+
+    def is_kvm_available(self):
+        """
+        Return True if kvm guests can be installed
+        """
+        for g in self.guests:
+            if g.os_type != "hvm":
+                continue
+
+            for d in g.domains:
+                if d.hypervisor_type == "kvm":
+                    return True
+
+        return False
+
+    def is_xenner_available(self):
+        """
+        Return True if xenner install option is available
+        """
+        for g in self.guests:
+            if g.os_type != "xen":
+                continue
+
+            for d in g.domains:
+                if d.hypervisor_type == "kvm":
+                    return True
+
+        return False
+
+    def is_bios_virt_disabled(self):
+        """
+        Try to determine if fullvirt may be disabled in the bios.
+
+        Check is basically:
+        - We support HW virt
+        - We appear to be xen
+        - There are no HVM install options
+
+        We don't do this check for KVM, since no KVM options may mean
+        KVM isn't installed or the module isn't loaded (and loading the
+        module will give an appropriate error
+        """
+        if not self.hw_virt_supported():
+            return False
+
+        if not self._is_xen():
+            return False
+
+        for g in self.guests:
+            if g.os_type == "hvm":
+                return False
+
+        return True
+
     def guestForOSType(self, type = None, arch = None):
         if self.host is None:
             return None
@@ -309,7 +468,7 @@ class Capabilities(object):
             if g.os_type != "hvm" or g.arch != "x86_64":
                 continue
             for d in g.domains:
-                if d.emulator.find("lib64") != -1:
+                if d.emulator is not None and d.emulator.find("lib64") != -1:
                     fixEmulator = d.emulator
 
         if not fixEmulator:
@@ -319,7 +478,7 @@ class Capabilities(object):
             if g.os_type != "hvm" or g.arch != "i686":
                 continue
             for d in g.domains:
-                if d.emulator.find("lib64") == -1:
+                if d.emulator is not None and d.emulator.find("lib64") == -1:
                     d.emulator = fixEmulator
 
     def parseXML(self, node):
