@@ -68,45 +68,26 @@ class FullVirtGuest(Guest):
         if (not self.loader) and self.type == "xen":
             self.loader = "/usr/lib/xen/boot/hvmloader"
 
+        # Add a default console device
+        dev = VirtualCharDevice.get_dev_instance(self.conn,
+                                                 VirtualCharDevice.DEV_CONSOLE,
+                                                 VirtualCharDevice.CHAR_PTY)
+        self.add_device(dev)
+        self._default_console_assigned = True
 
-    def os_features(self):
-        """Determine the guest features, based on explicit settings in FEATURES
-        and the OS_TYPE and OS_VARIANT. FEATURES takes precedence over the OS
-        preferences"""
-        if self.features is None:
-            return None
+        self._set_default_input_dev()
 
-        # explicitly disabling apic and acpi will override OS_TYPES values
-        features = dict(self.features)
-        for f in ["acpi", "apic"]:
-            val = self._lookup_osdict_key(f)
-            features[f] = val
-        return features
 
     def _get_input_device(self):
-        typ = self._lookup_device_param("input", "type")
-        bus = self._lookup_device_param("input", "bus")
+        inputtype = VirtualDevice.VIRTUAL_DEV_INPUT
+
+        typ = self._lookup_device_param(inputtype, "type")
+        bus = self._lookup_device_param(inputtype, "bus")
         dev = VirtualInputDevice(self.conn)
         dev.type = typ
         dev.bus = bus
         return dev
 
-    def _get_features_xml(self):
-        ret = "  <features>\n"
-        features = self.os_features()
-        if features:
-            ret += "    "
-            for k in sorted(features.keys()):
-                v = features[k]
-                if v:
-                    ret += "<%s/>" %(k,)
-            ret += "\n"
-        return ret + "  </features>"
-
-    def _get_clock_xml(self):
-        val = self._lookup_osdict_key("clock")
-        return """  <clock offset="%s"/>""" % val
-
     def _get_device_xml(self, install=True):
         emu_xml = ""
         if self.emulator is not None:
@@ -114,27 +95,24 @@ class FullVirtGuest(Guest):
 
         return (emu_xml + Guest._get_device_xml(self, install))
 
-    def _set_defaults(self):
-        disk_bus  = self._lookup_device_param("disk", "bus")
-        net_model = self._lookup_device_param("net", "model")
+    def _set_defaults(self, devlist_func):
+        disktype = VirtualDevice.VIRTUAL_DEV_DISK
+        nettype = VirtualDevice.VIRTUAL_DEV_NET
+        disk_bus  = self._lookup_device_param(disktype, "bus")
+        net_model = self._lookup_device_param(nettype, "model")
 
         # Only overwrite params if they weren't already specified
-        for net in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_NET):
+        for net in devlist_func(nettype):
             if net_model and not net.model:
                 net.model = net_model
-        for disk in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_DISK):
+
+        for disk in devlist_func(disktype):
             if (disk_bus and not disk.bus and
                 disk.device == VirtualDisk.DEVICE_DISK):
                 disk.bus = disk_bus
 
-        # If no serial devices were attached to the guest, stick the default
-        # console device in.
-        if (not self._get_install_devs(VirtualDevice.VIRTUAL_DEV_SERIAL) and
-            not self._get_install_devs(VirtualDevice.VIRTUAL_DEV_CONSOLE)):
-            dev = VirtualCharDevice.get_dev_instance(self.conn,
-                                                     VirtualCharDevice.DEV_CONSOLE,
-                                                     VirtualCharDevice.CHAR_PTY)
-            self._add_install_dev(dev)
+        if self.clock.offset == None:
+            self.clock.offset = self._lookup_osdict_key("clock")
 
         # Run this last, so we get first crack at disk attributes
-        Guest._set_defaults(self)
+        Guest._set_defaults(self, devlist_func)
