@@ -27,8 +27,11 @@ import _util
 import libvirt
 import CapabilitiesParser
 import VirtualGraphics
+import support
 from VirtualDevice import VirtualDevice
 from VirtualDisk import VirtualDisk
+from Clock import Clock
+from Seclabel import Seclabel
 
 import osdict
 from virtinst import _virtinst as _
@@ -108,9 +111,22 @@ class Guest(object):
 
     def __init__(self, type=None, connection=None, hypervisorURI=None,
                  installer=None):
+
+        # Set up the connection, since it is fundamental for other init
+        self.conn = connection
+        if self.conn == None:
+            logging.debug("No conn passed to Guest, opening URI '%s'" % \
+                          hypervisorURI)
+            self.conn = libvirt.open(hypervisorURI)
+
+        if self.conn == None:
+            raise RuntimeError, _("Unable to connect to hypervisor, aborting "
+                                  "installation!")
+
         # We specifically ignore the 'type' parameter here, since
         # it has been replaced by installer.type, and child classes can
         # use it when creating a default installer.
+        ignore = type
         self._installer = installer
         self._name = None
         self._uuid = None
@@ -119,11 +135,16 @@ class Guest(object):
         self._vcpus = 1
         self._cpuset = None
         self._graphics_dev = None
-        self._consolechild = None
-        self._os_autodetect = False
+        self._autostart = False
+        self._clock = Clock(self.conn)
+        self._seclabel = None
+        self._description = None
+        self.features = None
+        self._replace = None
 
         self._os_type = None
         self._os_variant = None
+        self._os_autodetect = False
 
         # DEPRECATED: Public device lists unaltered by install process
         self.disks = []
@@ -140,6 +161,7 @@ class Guest(object):
 
         # The libvirt virDomain object we 'Create'
         self.domain = None
+        self._consolechild = None
 
         # Default disk target prefix ('hd' or 'xvd'). Set in subclass
         self.disknode = None
@@ -147,38 +169,59 @@ class Guest(object):
         # Default bus for disks (set in subclass)
         self._diskbus = None
 
-        self.conn = connection
-        if self.conn == None:
-            logging.debug("No conn passed to Guest, opening URI '%s'" % \
-                          hypervisorURI)
-            self.conn = libvirt.open(hypervisorURI)
-        if self.conn == None:
-            raise RuntimeError, _("Unable to connect to hypervisor, aborting "
-                                  "installation!")
+        # Indicates that default devices have been assigned, so look for
+        # the user to overwrite
+        self._default_console_assigned = None
+        self._default_input_assigned = None
 
         self._caps = CapabilitiesParser.parse(self.conn.getCapabilities())
 
 
+    ######################
+    # Property accessors #
+    ######################
+
     def get_installer(self):
         return self._installer
     def set_installer(self, val):
-        # FIXME: Make sure this is valid: it's pretty fundamental to
-        # working operation. Should we even allow it to be changed?
         self._installer = val
     installer = property(get_installer, set_installer)
 
+    def get_clock(self):
+        return self._clock
+    clock = property(get_clock)
+
+    def get_seclabel(self):
+        return self._seclabel
+    def set_seclabel(self, val):
+        if val and not isinstance(val, Seclabel):
+            raise ValueError("'seclabel' must be a Seclabel() instance.")
+
+        if val:
+            # Check for validation purposes
+            val.get_xml_config()
+        self._seclabel = val
+    seclabel = property(get_seclabel, set_seclabel)
+
     # Domain name of the guest
     def get_name(self):
         return self._name
     def set_name(self, val):
         _util.validate_name(_("Guest"), val)
-        try:
-            self.conn.lookupByName(val)
-        except:
-            # Name not found
-            self._name = val
-            return
-        raise ValueError(_("Guest name '%s' is already in use.") % val)
+
+        do_fail = False
+        if self.replace != True:
+            try:
+                self.conn.lookupByName(val)
+                do_fail = True
+            except:
+                # Name not found
+                pass
+
+        if do_fail:
+            raise ValueError(_("Guest name '%s' is already in use.") % val)
+
+        self._name = val
     name = property(get_name, set_name)
 
     # Memory allocated to the guest.  Should be given in MB
@@ -255,16 +298,16 @@ class Guest(object):
         val = val.lower()
 
         if self._OS_TYPES.has_key(val):
-            if self._os_type == val:
-                # No change, don't invalidate variant
-                return
-
-            # Invalidate variant, since it may not apply to the new os type
-            self._os_type = val
-            self._os_variant = None
+            if self._os_type != val:
+                # Invalidate variant, since it may not apply to the new os type
+                self._os_type = val
+                self._os_variant = None
         else:
             raise ValueError, _("OS type '%s' does not exist in our "
                                 "dictionary") % val
+
+        # Default may have changed with OS info
+        self._set_default_input_dev()
     os_type = property(get_os_type, set_os_type)
 
     def get_os_variant(self):
@@ -273,6 +316,7 @@ class Guest(object):
         if type(val) is not str:
             raise ValueError(_("OS variant must be a string."))
         val = val.lower()
+
         if self.os_type:
             if self._OS_TYPES[self.os_type]["variants"].has_key(val):
                 self._os_variant = val
@@ -281,6 +325,7 @@ class Guest(object):
                                     "our dictionary for OS type '%(ty)s'" ) % \
                                     {'var' : val, 'ty' : self._os_type}
         else:
+            found = False
             for ostype in self.list_os_types():
                 if self._OS_TYPES[ostype]["variants"].has_key(val) and \
                    not self._OS_TYPES[ostype]["variants"][val].get("skip"):
@@ -288,8 +333,13 @@ class Guest(object):
                                   (ostype, val))
                     self.os_type = ostype
                     self._os_variant = val
-                    return
-            raise ValueError, _("Unknown OS variant '%s'" % val)
+                    found = True
+
+            if not found:
+                raise ValueError, _("Unknown OS variant '%s'" % val)
+
+        # Default may have changed with OS info
+        self._set_default_input_dev()
     os_variant = property(get_os_variant, set_os_variant)
 
     def set_os_autodetect(self, val):
@@ -303,8 +353,30 @@ class Guest(object):
         return self._lookup_osdict_key("distro")
     os_distro = property(get_os_distro)
 
+    def get_autostart(self):
+        return self._autostart
+    def set_autostart(self, val):
+        self._autostart = bool(val)
+    autostart = property(get_autostart, set_autostart,
+                         doc="Have domain autostart when the host boots.")
 
-    # DEPRECATED PROPERTIES
+    def _get_description(self):
+        return self._description
+    def _set_description(self, val):
+        self._description = val
+    description = property(_get_description, _set_description)
+
+    def _get_replace(self):
+        return self._replace
+    def _set_replace(self, val):
+        self._replace = bool(val)
+    replace = property(_get_replace, _set_replace,
+                       doc=_("Whether we should overwrite an existing guest "
+                             "with the same name."))
+
+    #########################
+    # DEPRECATED PROPERTIES #
+    #########################
 
     # Deprecated: Should set graphics_dev.keymap directly
     def get_keymap(self):
@@ -371,16 +443,15 @@ class Guest(object):
 
     graphics = property(get_graphics, set_graphics)
 
-
-    # Properties that are mapped through to the Installer
-
     # Hypervisor name (qemu, xen, kvm, etc.)
+    # Deprecated: should be pulled directly from the installer
     def get_type(self):
         return self._installer.type
     def set_type(self, val):
         self._installer.type = val
     type = property(get_type, set_type)
 
+    # Deprecated: should be pulled directly from the installer
     def get_arch(self):
         return self.installer.arch
     def set_arch(self, val):
@@ -417,21 +488,14 @@ class Guest(object):
     def get_cdrom(self):
         return self._installer.location
     def set_cdrom(self, val):
-        if val is None or type(val) is not type("string") or len(val) == 0:
-            raise ValueError, _("You must specify a valid ISO or CD-ROM location for the installation")
-        if val.startswith("/"):
-            if not os.path.exists(val):
-                raise ValueError, _("The specified media path does not exist.")
-            self._installer.location = os.path.abspath(val)
-        else:
-            # Assume its a http/nfs/ftp style path
-            self._installer.location = val
+        self._installer.location = val
         self._installer.cdrom = True
     cdrom = property(get_cdrom, set_cdrom)
-    # END DEPRECATED PROPERTIES
 
 
-    # Device Add/Remove Public API methods
+    ########################################
+    # Device Add/Remove Public API methods #
+    ########################################
 
     def _dev_build_list(self, devtype, devlist=None):
         if not devlist:
@@ -453,6 +517,23 @@ class Guest(object):
             raise ValueError(_("Must pass a VirtualDevice instance."))
         devtype = dev.virtual_device_type
 
+        # Handling for back compat default device behavior
+        if ((devtype == VirtualDevice.VIRTUAL_DEV_CONSOLE or
+             devtype == VirtualDevice.VIRTUAL_DEV_SERIAL) and
+            self._default_console_assigned == True):
+
+            rmdev = self.get_devices(VirtualDevice.VIRTUAL_DEV_CONSOLE)[0]
+            self.remove_device(rmdev)
+            self._default_console_assigned = False
+
+        if (devtype == VirtualDevice.VIRTUAL_DEV_INPUT and
+            self._default_input_assigned == True):
+
+            rmdev = self.get_devices(VirtualDevice.VIRTUAL_DEV_INPUT)[0]
+            self.remove_device(rmdev)
+            self._default_input_assigned = False
+
+        # Actually add the device
         if   devtype == VirtualDevice.VIRTUAL_DEV_DISK:
             self.disks.append(dev)
         elif devtype == VirtualDevice.VIRTUAL_DEV_NET:
@@ -502,20 +583,36 @@ class Guest(object):
 
         @param dev: VirtualDevice instance
         """
+        found = False
         if dev == self._graphics_dev:
             self._graphics_dev = None
+            found = True
 
         for devlist in [self.disks, self.nics, self.sound_devs, self.hostdevs,
                         self._devices]:
+            if found:
+                break
+
             if dev in devlist:
                 devlist.remove(dev)
+                found = True
+                break
+
+        if not found:
+            raise ValueError(_("Did not find device %s") % str(dev))
+
+        if (dev.virtual_device_type == VirtualDevice.VIRTUAL_DEV_INPUT and
+            self._default_input_assigned == True):
+            self._default_input_assigned = False
+        if (dev.virtual_device_type in [VirtualDevice.VIRTUAL_DEV_CONSOLE,
+                                        VirtualDevice.VIRTUAL_DEV_SERIAL] and
+            self._default_console_assigned == True):
+            self._default_console_assigned = False
+
 
     # Device fetching functions used internally during the install process.
     # These allow us to change dev defaults, add install media, etc. during
     # the install, but revert to a clean state if the install fails
-    def _init_install_devs(self):
-        self._install_devices = self.get_all_devices()[:]
-
     def _get_install_devs(self, devtype):
         return self._dev_build_list(devtype, self._install_devices)
 
@@ -528,7 +625,10 @@ class Guest(object):
             retlist.extend(self._get_install_devs(devtype))
         return retlist
 
-    # Private xml building methods
+
+    ################################
+    # Private xml building methods #
+    ################################
 
     def _get_input_device(self):
         """ Return a tuple of the form (devtype, bus) for the desired
@@ -536,45 +636,113 @@ class Guest(object):
         raise NotImplementedError
 
     def _get_device_xml(self, install=True):
-        change_disks = []
-        for d in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_DISK):
-            if (d.device == VirtualDisk.DEVICE_CDROM
-                and d.transient
-                and not install):
-                # Keep cdrom around, but with no media attached,
-                # But only if we are a distro that doesn't have a multi
-                # stage install (aka not Windows)
-                if not self.get_continue_inst():
-                    change_disks.append((d, d.path))
-                    d.path = None
+        # If install hasn't been prepared yet, make sure we use
+        # the regular device list
+        inst_devs = self._get_all_install_devs()
+        reg_devs = self.get_all_devices()
+        if len(inst_devs) >= len(reg_devs):
+            devs = inst_devs[:]
+        else:
+            devs = reg_devs[:]
+
+        def do_remove_media(d):
+            # Keep cdrom around, but with no media attached,
+            # But only if we are a distro that doesn't have a multi
+            # stage install (aka not Windows)
+            return (d.virtual_device_type == VirtualDevice.VIRTUAL_DEV_DISK and
+                    d.device == VirtualDisk.DEVICE_CDROM
+                    and d.transient
+                    and not install and
+                    not self.get_continue_inst())
+
+        # Wrapper for building disk XML, handling transient CDROMs
+        def get_dev_xml(dev):
+            origpath = None
+            try:
+                if do_remove_media(dev):
+                    origpath = dev.path
+                    dev.path = None
+
+                return dev.get_xml_config()
+            finally:
+                if origpath:
+                    dev.path = origpath
 
         xml = ""
-        try:
-            for dev in self._get_all_install_devs():
-                xml = _util.xml_append(xml, dev.get_xml_config())
-        finally:
-            try:
-                for disk, path in change_disks:
-                    disk.path = path
-            except:
-                pass
+        # Build XML
+        for dev in devs:
+            xml = _util.xml_append(xml, get_dev_xml(dev))
 
         return xml
 
+    def _get_features(self):
+        """
+        Determine the guest features, based on explicit settings in FEATURES
+        and the OS_TYPE and OS_VARIANT. FEATURES takes precedence over the OS
+        preferences
+        """
+        if self.features is None:
+            return None
+
+        # explicitly disabling apic and acpi will override OS_TYPES values
+        features = dict(self.features)
+        for f in ["acpi", "apic"]:
+            val = self._lookup_osdict_key(f)
+            if features.get(f) == None:
+                features[f] = val
+        return features
+
     def _get_features_xml(self):
         """
-        Return features (pae, acpi, apic) xml (currently only releavnt for FV)
+        Return features (pae, acpi, apic) xml
         """
+        features = self._get_features()
+        found_feature = False
+
+        ret = "  <features>\n"
+
+        if features:
+            ret += "    "
+            for k in sorted(features.keys()):
+                v = features[k]
+                if v:
+                    ret += "<%s/>" % k
+                    found_feature = True
+            ret += "\n"
+
+        if not found_feature:
+            # Don't print empty feature block if no features added
+            return ""
+
+        return ret + "  </features>"
+
+    def _get_cpu_xml(self):
+        """
+        Return <cpu> XML
+        """
+        # Just a stub for now
         return ""
 
     def _get_clock_xml(self):
         """
-        Return <clock/> xml (currently only relevant for FV guests)
+        Return <clock/> xml
         """
-        return ""
+        return self.clock.get_xml_config()
+
+    def _get_seclabel_xml(self):
+        """
+        Return <seclabel> XML
+        """
+        xml = ""
+        if self.seclabel:
+            xml = self.seclabel.get_xml_config()
+
+        return xml
 
     def _get_osblob(self, install):
-        """Return os, features, and clock xml (Implemented in subclass)"""
+        """
+        Return os, features, and clock xml (Implemented in subclass)
+        """
         xml = ""
 
         osxml = self.installer.get_install_xml(self, install)
@@ -583,11 +751,44 @@ class Guest(object):
 
         xml = _util.xml_append(xml,
                                self.installer.get_install_xml(self, install))
-        xml = _util.xml_append(xml, self._get_features_xml())
-        xml = _util.xml_append(xml, self._get_clock_xml())
         return xml
 
+    ############################
+    # Install Helper functions #
+    ############################
 
+    def _prepare_install(self, meter):
+        # Initialize install device list
+        self._install_devices = self.get_all_devices()[:]
+
+        # Set regular device defaults
+        self.set_defaults()
+
+        self._installer.prepare(guest = self,
+                                meter = meter)
+        if self._installer.install_disk is not None:
+            self._add_install_dev(self._installer.install_disk)
+
+        # Run 'set_defaults' after install prep, since some installers
+        # (ImageInstaller) alter the device list.
+        self._set_defaults(self._get_install_devs)
+
+    def _cleanup_install(self):
+        # Empty install dev list
+        self._install_devices = []
+
+        self._installer.cleanup()
+
+    def _create_devices(self, progresscb):
+        """
+        Ensure that devices are setup
+        """
+        for dev in self._get_all_install_devs():
+            dev.setup_dev(self.conn, progresscb)
+
+    ##############
+    # Public API #
+    ##############
 
     def get_config_xml(self, install = True, disk_boot = False):
         """
@@ -606,6 +807,9 @@ class Guest(object):
         @type disk_boot: C{bool}
         """
 
+        # Set device defaults so we can validly generate XML
+        self._set_defaults(self.get_devices)
+
         if install:
             action = "destroy"
         else:
@@ -620,10 +824,15 @@ class Guest(object):
             # This means there is no 'install' phase, so just return
             return None
 
+        cpuset = ""
         if self.cpuset is not None:
             cpuset = " cpuset='" + self.cpuset + "'"
-        else:
-            cpuset = ""
+
+        desc_xml = ""
+        if self.description is not None:
+            desc = str(self.description)
+            desc_xml = ("  <description>%s</description>" %
+                        _util.xml_escape(desc))
 
         xml = ""
         add = lambda x: _util.xml_append(xml, x)
@@ -633,35 +842,34 @@ class Guest(object):
         xml = add("  <currentMemory>%s</currentMemory>" % (self.memory * 1024))
         xml = add("  <memory>%s</memory>" % (self.maxmemory * 1024))
         xml = add("  <uuid>%s</uuid>" % self.uuid)
+        xml = add(desc_xml)
         xml = add("  %s" % osblob)
+        xml = add(self._get_features_xml())
+        xml = add(self._get_cpu_xml())
+        xml = add(self._get_clock_xml())
         xml = add("  <on_poweroff>destroy</on_poweroff>")
         xml = add("  <on_reboot>%s</on_reboot>" % action)
         xml = add("  <on_crash>%s</on_crash>" % action)
         xml = add("  <vcpu%s>%d</vcpu>" % (cpuset, self.vcpus))
         xml = add("  <devices>")
-        xml = add("%s" % self._get_device_xml(install))
+        xml = add(self._get_device_xml(install))
         xml = add("  </devices>")
+        xml = add(self._get_seclabel_xml())
         xml = add("</domain>\n")
 
         return xml
 
-    def start_install(self, consolecb=None, meter=None, removeOld=False,
-                      wait=True):
-        """Do the startup of the guest installation."""
-        self.validate_parms()
-        self._consolechild = None
-
-        if meter is None:
-            # BaseMeter does nothing, but saves a lot of null checking
-            meter = progress.BaseMeter()
-
-        self._prepare_install(meter)
-        try:
-            return self._do_install(consolecb, meter, removeOld, wait)
-        finally:
-            self._installer.cleanup()
+    def post_install_check(self):
+        """
+        Back compat mapping to installer post_install_check
+        """
+        return self.installer.post_install_check(self)
 
     def get_continue_inst(self):
+        """
+        Return True if this guest requires a call to 'continue_install',
+        which means the OS requires a 2 stage install (windows)
+        """
         val = self._lookup_osdict_key("continue")
         if not val:
             val = False
@@ -673,156 +881,283 @@ class Guest(object):
                 val = False
         return val
 
-    def continue_install(self, consolecb, meter, wait=True):
-        cont_xml = self.get_config_xml(disk_boot = True)
-        logging.debug("Continuing guest with:\n%s" % cont_xml)
-        meter.start(size=None, text="Starting domain...")
+    def validate_parms(self):
+        """
+        Do some pre-install domain validation
+        """
+        if self.domain is not None:
+            raise RuntimeError, _("Domain has already been started!")
 
-        # As of libvirt 0.5.1 we can't 'create' over an defined VM.
-        # So, redefine the existing domain (which should be shutoff at
-        # this point), and start it.
-        finalxml = self.domain.XMLDesc(0)
+        if self.name is None or self.memory is None:
+            raise RuntimeError(_("Name and memory must be specified for "
+                                 "all guests!"))
 
-        self.domain = self.conn.defineXML(cont_xml)
-        self.domain.create()
-        self.conn.defineXML(finalxml)
+        if _util.vm_uuid_collision(self.conn, self.uuid):
+            raise RuntimeError(_("The UUID you entered is already in "
+                                 "use by another guest!"))
 
-        #self.domain = self.conn.createLinux(install_xml, 0)
-        if self.domain is None:
-            raise RuntimeError, _("Unable to start domain for guest, aborting installation!")
-        meter.end(0)
+    def connect_console(self, consolecb, wait=True):
+        """
+        Launched the passed console callback for the already defined
+        domain. If domain isn't running, return an error.
+        """
+        (self.domain,
+         self._consolechild) = self._wait_and_connect_console(consolecb)
 
-        self.connect_console(consolecb, wait)
+        # If we connected the console, wait for it to finish
+        self._waitpid_console(self._consolechild, wait)
+
+    def terminate_console(self):
+        """
+        Kill guest console if it is open (and actually exists), otherwise
+        do nothing
+        """
+        if self._consolechild:
+            try:
+                os.kill(self._consolechild, signal.SIGKILL)
+            except:
+                pass
+
+    def domain_is_shutdown(self):
+        """
+        Return True if the created domain object is shutdown
+        """
+        dom = self.domain
+        if not dom:
+            return False
+
+        dominfo = dom.info()
+
+        state    = dominfo[0]
+        cpu_time = dominfo[4]
+
+        if state == libvirt.VIR_DOMAIN_SHUTOFF:
+            return True
+
+        # If 'wait' was specified, the dom object we have was looked up
+        # before initially shutting down, which seems to bogus up the
+        # info data (all 0's). So, if it is bogus, assume the domain is
+        # shutdown. We will catch the error later.
+        return state == libvirt.VIR_DOMAIN_NOSTATE and cpu_time == 0
+
+    def domain_is_crashed(self):
+        """
+        Return True if the created domain object is in a crashed state
+        """
+        if not self.domain:
+            return False
+
+        dominfo = self.domain.info()
+        state = dominfo[0]
+
+        return state == libvirt.VIR_DOMAIN_CRASHED
+
+    ##########################
+    # Actual install methods #
+    ##########################
+
+    def start_install(self, consolecb=None, meter=None, removeOld=None,
+                      wait=True):
+        """
+        Begin the guest install (stage1).
+        """
+        if removeOld == None:
+            removeOld = self.replace
+
+        self.validate_parms()
+        self._consolechild = None
+
+        self._prepare_install(meter)
+        try:
+            return self._do_install(consolecb, meter, removeOld, wait)
+        finally:
+            self._cleanup_install()
+
+    def continue_install(self, consolecb=None, meter=None, wait=True):
+        """
+        Continue with stage 2 of a guest install. Only required for
+        guests which have the 'continue' flag set (accessed via
+        get_continue_inst)
+        """
+        return self._create_guest(consolecb, meter, wait,
+                                  _("Starting domain..."), "continue",
+                                  disk_boot=True,
+                                  first_create=False)
+
+    def _create_guest(self, consolecb, meter, wait,
+                      meter_label, log_label,
+                      disk_boot=False,
+                      first_create=True):
+        """
+        Actually do the XML logging, guest defining/creating, console
+        launching and waiting
+        """
+        if meter == None:
+            meter = progress.BaseMeter()
+
+        start_xml = self.get_config_xml(install=True, disk_boot=disk_boot)
+        final_xml = self.get_config_xml(install=False)
+        logging.debug("Generated %s XML: %s" %
+                      (log_label,
+                      (start_xml and ("\n" + start_xml) or "None required")))
+
+        if start_xml:
+            meter.start(size=None, text=meter_label)
+
+            if first_create:
+                dom = self.conn.createLinux(start_xml, 0)
+            else:
+                dom = self.conn.defineXML(start_xml)
+                dom.create()
+
+            self.domain = dom
+            meter.end(0)
+
+            logging.debug("Started guest, looking to see if it is running")
+            (self.domain,
+             self._consolechild) = self._wait_and_connect_console(consolecb)
+
+        logging.debug("Generated boot XML: \n%s" % final_xml)
+        self.domain = self.conn.defineXML(final_xml)
+
+        # if we connected the console, wait for it to finish
+        self._waitpid_console(self._consolechild, wait)
+
+        return self.conn.lookupByName(self.name)
+
+    def _do_install(self, consolecb, meter, removeOld=False, wait=True):
+        # Remove existing VM if requested
+        self._replace_original_vm(removeOld)
+
+        # Create devices if required (disk images, etc.)
+        self._create_devices(meter)
+
+        self.domain = self._create_guest(consolecb, meter, wait,
+                                         _("Creating domain..."),
+                                         "install")
+
+        # Set domain autostart flag if requested
+        self._flag_autostart()
+
+        return self.domain
+
+    def _wait_and_connect_console(self, consolecb):
+        """
+        Wait for domain to appear and be running, then connect to
+        the console if necessary
+        """
+        child = None
+        dom = _wait_for_domain(self.conn, self.name)
+
+        if dom is None:
+            raise RuntimeError(_("Domain has not existed.  You should be "
+                                 "able to find more information in the logs"))
+        elif dom.ID() == -1:
+            raise RuntimeError(_("Domain has not run yet.  You should be "
+                                 "able to find more information in the logs"))
+
+        if consolecb:
+            logging.debug("Launching console callback")
+            child = consolecb(dom)
+
+        return dom, child
+
+    def _waitpid_console(self, console_child, do_wait):
+        """
+        Wait for console to close if it was launched
+        """
+        if not console_child or not do_wait:
+            return
+
+        try:
+            os.waitpid(console_child, 0)
+        except OSError, (err_no, msg):
+            logging.debug("waitpid: %s: %s" % (err_no, msg))
 
         # ensure there's time for the domain to finish destroying if the
         # install has finished or the guest crashed
-        if consolecb:
-            time.sleep(1)
+        time.sleep(1)
 
-        # This should always work, because it'll lookup a config file
-        # for inactive guest, or get the still running install..
-        return self.conn.lookupByName(self.name)
-
-
-    def _prepare_install(self, meter):
-        self._init_install_devs()
-
-        self._installer.prepare(guest = self,
-                                meter = meter)
-        if self._installer.install_disk is not None:
-            self._add_install_dev(self._installer.install_disk)
-
-        # Run 'set_defaults' after install prep, since some installers
-        # (ImageInstaller) alter the device list.
-        self._set_defaults()
-
-        # Only set up an input device if there isn't already preset
-        if not self._get_install_devs(VirtualDevice.VIRTUAL_DEV_INPUT):
-            self._add_install_dev(self._get_input_device())
-
-    def _create_devices(self, progresscb):
-        """Ensure that devices are setup"""
-        for disk in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_DISK):
-            disk.setup(progresscb)
-        for nic in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_NET):
-            nic.setup(self.conn)
-        for hostdev in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_HOSTDEV):
-            hostdev.setup()
-
-    def _do_install(self, consolecb, meter, removeOld=False, wait=True):
+    def _replace_original_vm(self, removeOld):
+        """
+        Remove the existing VM with the same name if requested, or error
+        if there is a collision.
+        """
         vm = None
         try:
             vm = self.conn.lookupByName(self.name)
         except libvirt.libvirtError:
             pass
 
-        if vm is not None:
-            if removeOld :
-                try:
-                    if vm.ID() != -1:
-                        logging.info("Destroying image %s" %(self.name))
-                        vm.destroy()
-                    logging.info("Removing old definition for image %s" %(self.name))
-                    vm.undefine()
-                except libvirt.libvirtError, e:
-                    raise RuntimeError, _("Could not remove old vm '%s': %s") %(self.name, str(e))
+        if vm is None:
+            return
+
+        if not removeOld:
+            raise RuntimeError(_("Domain named %s already exists!") %
+                               self.name)
+
+        try:
+            if vm.ID() != -1:
+                logging.info("Destroying image %s" % self.name)
+                vm.destroy()
+
+            logging.info("Removing old definition for image %s" % self.name)
+            vm.undefine()
+        except libvirt.libvirtError, e:
+            raise RuntimeError(_("Could not remove old vm '%s': %s") %
+                               (self.name, str(e)))
+
+
+    def _flag_autostart(self):
+        """
+        Set the autostart flag for self.domain if the user requested it
+        """
+        if not self.autostart:
+            return
+
+        try:
+            self.domain.setAutostart(True)
+        except libvirt.libvirtError, e:
+            if support.is_error_nosupport(e):
+                logging.warn("Could not set autostart flag: libvirt "
+                             "connection does not support autostart.")
             else:
-                raise RuntimeError, _("Domain named %s already exists!") %(self.name,)
+                raise e
 
-        child = None
-        self._create_devices(meter)
-        install_xml = self.get_config_xml()
-        if install_xml:
-            logging.debug("Creating guest from:\n%s" % install_xml)
-            meter.start(size=None, text=_("Creating domain..."))
-            self.domain = self.conn.createLinux(install_xml, 0)
-            if self.domain is None:
-                raise RuntimeError, _("Unable to create domain for the guest, aborting installation!")
-            meter.end(0)
 
-            logging.debug("Created guest, looking to see if it is running")
+    ###################
+    # Device defaults #
+    ###################
 
-            d = _wait_for_domain(self.conn, self.name)
+    def _set_default_input_dev(self):
+        # This is called at init time, but also whenever the OS changes,
+        # since the input dev maybe be dependent on OS
+        if self._default_input_assigned == False:
+            return
 
-            if d is None:
-                raise RuntimeError, _("It appears that your installation has crashed.  You should be able to find more information in the logs")
+        for d in self.get_devices(VirtualDevice.VIRTUAL_DEV_INPUT):
+            self.remove_device(d)
 
-            if consolecb:
-                logging.debug("Launching console callback")
-                child = consolecb(self.domain)
-                self._consolechild = child
+        # Add default input device
+        self._default_input_assigned = False
+        self.add_device(self._get_input_device())
+        self._default_input_assigned = True
 
-        boot_xml = self.get_config_xml(install = False)
-        logging.debug("Saving XML boot config:\n%s" % boot_xml)
-        self.conn.defineXML(boot_xml)
+    def set_defaults(self):
+        """
+        Public function to set guest defaults. Things like preferred
+        disk bus (unless one is specified). This will be called by the
+        install process, but can be called earlier if needed
+        """
+        self._set_defaults(self.get_devices)
 
-        if child and wait: # if we connected the console, wait for it to finish
-            try:
-                os.waitpid(child, 0)
-            except OSError, (err_no, msg):
-                print __name__, "waitpid: %s: %s" % (err_no, msg)
+    def _set_defaults(self, devlist_func):
+        soundtype = VirtualDevice.VIRTUAL_DEV_AUDIO
+        videotype = VirtualDevice.VIRTUAL_DEV_VIDEO
 
-            # ensure there's time for the domain to finish destroying if the
-            # install has finished or the guest crashed
-            time.sleep(1)
-
-        # This should always work, because it'll lookup a config file
-        # for inactive guest, or get the still running install..
-        return self.conn.lookupByName(self.name)
-
-    def post_install_check(self):
-        return self.installer.post_install_check(self)
-
-    def connect_console(self, consolecb, wait=True):
-        logging.debug("Restarted guest, looking to see if it is running")
-
-        self.domain = _wait_for_domain(self.conn, self.name)
-
-        if self.domain is None:
-            raise RuntimeError, _("Domain has not existed.  You should be able to find more information in the logs")
-        elif self.domain.ID() == -1:
-            raise RuntimeError, _("Domain has not run yet.  You should be able to find more information in the logs")
-
-        child = None
-        if consolecb:
-            logging.debug("Launching console callback")
-            child = consolecb(self.domain)
-            self._consolechild = child
-
-        if child and wait: # if we connected the console, wait for it to finish
-            try:
-                os.waitpid(child, 0)
-            except OSError, (err_no, msg):
-                raise RuntimeError, \
-                      "waiting console pid error: %s: %s" % (err_no, msg)
-
-    def validate_parms(self):
-        if self.domain is not None:
-            raise RuntimeError, _("Domain has already been started!")
-
-    def _set_defaults(self):
+        # Generate disk targets, and set preferred disk bus
         used_targets = []
-        for disk in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_DISK):
+        for disk in devlist_func(VirtualDevice.VIRTUAL_DEV_DISK):
             if not disk.bus:
                 if disk.device == disk.DEVICE_FLOPPY:
                     disk.bus = "fdc"
@@ -830,20 +1165,30 @@ class Guest(object):
                     disk.bus = self._diskbus
             used_targets.append(disk.generate_target(used_targets))
 
+        # Set sound device model
+        sound_model  = self._lookup_device_param(soundtype, "model")
+        for sound in devlist_func(soundtype):
+            if sound.model == sound.MODEL_DEFAULT:
+                sound.model = sound_model
+
+        # Set video device model
+        video_model  = self._lookup_device_param(videotype, "model_type")
+        for video in devlist_func(videotype):
+            if video.model_type == video.MODEL_DEFAULT:
+                video.model_type = video_model
+
+        # Generate UUID
         if self.uuid is None:
             while 1:
                 self.uuid = _util.uuidToString(_util.randomUUID())
                 if _util.vm_uuid_collision(self.conn, self.uuid):
                     continue
                 break
-        else:
-            if _util.vm_uuid_collision(self.conn, self.uuid):
-                raise RuntimeError, _("The UUID you entered is already in "
-                                      "use by another guest!")
-        if self.name is None or self.memory is None:
-            raise RuntimeError, _("Name and memory must be specified for all guests!")
 
-    # Guest Dictionary Helper methods
+
+    ###################################
+    # Guest Dictionary Helper methods #
+    ###################################
 
     def _lookup_osdict_key(self, key):
         """
@@ -861,12 +1206,6 @@ class Guest(object):
         return osdict.lookup_device_param(self.conn, self.type, self.os_type,
                                           self.os_variant, device_key, param)
 
-    def terminate_console(self):
-        if self._consolechild:
-            try:
-                os.kill(self._consolechild, signal.SIGKILL)
-            except:
-                pass
 
 def _wait_for_domain(conn, name):
     # sleep in .25 second increments until either a) we get running
