@@ -50,7 +50,7 @@ import threading
 import time
 
 import logging
-from xml.sax.saxutils import escape
+from _util import xml_escape as escape
 
 import _util
 import support
@@ -290,7 +290,10 @@ class StoragePool(StorageObject):
 
     def get_pool_type_desc(pool_type):
         """Return human readable description for passed pool type"""
-        return StoragePool._types[pool_type]
+        if StoragePool._types.has_key(pool_type):
+            return StoragePool._types[pool_type]
+        else:
+            return "%s pool" % pool_type
     get_pool_type_desc = staticmethod(get_pool_type_desc)
 
     def pool_list_from_sources(conn, name, pool_type, host=None):
@@ -1143,7 +1146,7 @@ class StorageVolume(StorageObject):
                 self._install_finished = False
                 t.start()
                 if meter:
-                    meter.start(size=self.allocation,
+                    meter.start(size=self.capacity,
                                 text=_("Allocating '%s'") % self.name)
 
                 if self.input_vol:
@@ -1152,7 +1155,7 @@ class StorageVolume(StorageObject):
                     vol = self.pool.createXML(xml, 0)
 
                 if meter:
-                    meter.end(self.allocation)
+                    meter.end(self.capacity)
                 logging.debug("Storage volume '%s' install complete." %
                               self.name)
                 return vol
