@@ -99,10 +99,14 @@ def _is_dir_searchable(uid, username, path):
 
     # Check POSIX ACL (since that is what we use to 'fix' access)
     cmd = ["getfacl", path]
-    proc = subprocess.Popen(cmd,
-                            stdout=subprocess.PIPE,
-                            stderr=subprocess.PIPE)
-    out, err = proc.communicate()
+    try:
+        proc = subprocess.Popen(cmd,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE)
+        out, err = proc.communicate()
+    except OSError:
+        logging.debug("Didn't find the getfacl command.")
+        return False
 
     if proc.returncode != 0:
         logging.debug("Cmd '%s' failed: %s" % (cmd, err))
@@ -173,7 +177,8 @@ class VirtualDisk(VirtualDevice):
 
     TYPE_FILE = "file"
     TYPE_BLOCK = "block"
-    types = [TYPE_FILE, TYPE_BLOCK]
+    TYPE_DIR = "dir"
+    types = [TYPE_FILE, TYPE_BLOCK, TYPE_DIR]
 
     @staticmethod
     def path_exists(conn, path):
@@ -282,12 +287,68 @@ class VirtualDisk(VirtualDevice):
 
         return errdict
 
+    @staticmethod
+    def path_in_use_by(conn, path, check_conflict=False):
+        """
+        Return a list of VM names that are using the passed path.
+
+        @param conn: virConnect to check VMs
+        @param path: Path to check for
+        @param check_conflict: Only return names that are truly conflicting:
+                               this will omit guests that are using the disk
+                               with the 'shareable' flag, and possible other
+                               heuristics
+        """
+        if not path:
+            return
+
+        active, inactive = _util.fetch_all_guests(conn)
+        vms = active + inactive
+
+        def count_cb(ctx):
+            c = 0
+
+            template = "count(/domain/devices/disk["
+            if check_conflict:
+                template += "not(shareable) and "
+            template += "source/@%s='%s'])"
+
+            for dtype in ["dev", "file", "dir"]:
+                xpath = template % (dtype, path)
+                c += ctx.xpathEval(xpath)
+
+            return c
+
+        names = []
+        for vm in vms:
+            xml = vm.XMLDesc(0)
+            tmpcount = _util.get_xml_path(xml, func = count_cb)
+            if tmpcount:
+                names.append(vm.name())
+
+        return names
+
+    @staticmethod
+    def stat_local_path(path):
+        """
+        Return tuple (storage type, storage size) for the passed path on
+        the local machine. This is a best effort attempt.
+
+        @return: tuple of
+                 (True if regular file, False otherwise, default is True,
+                  max size of storage, default is 0)
+        """
+        try:
+            return _util.stat_disk(path)
+        except:
+            return (True, 0)
 
     def __init__(self, path=None, size=None, transient=False, type=None,
                  device=DEVICE_DISK, driverName=None, driverType=None,
                  readOnly=False, sparse=True, conn=None, volObject=None,
                  volInstall=None, volName=None, bus=None, shareable=False,
-                 driverCache=None, selinuxLabel=None, format=None):
+                 driverCache=None, selinuxLabel=None, format=None,
+                 validate=True):
         """
         @param path: filesystem path to the disk image.
         @type path: C{str}
@@ -326,6 +387,10 @@ class VirtualDisk(VirtualDevice):
         @type selinuxLabel: C{str}
         @param format: Storage volume format to use when creating storage
         @type format: C{str}
+        @param validate: Whether to validate passed parameters against the
+                         local system. Omitting this may cause issues, be
+                         warned!
+        @type validate: C{bool}
         """
 
         VirtualDevice.__init__(self, conn=conn)
@@ -344,6 +409,7 @@ class VirtualDisk(VirtualDevice):
         self._selinux_label = None
         self._clone_path = None
         self._format = None
+        self._validate = validate
 
         # XXX: No property methods for these
         self.transient = transient
@@ -385,6 +451,12 @@ class VirtualDisk(VirtualDevice):
         if val is not None:
             self._check_str(val, "path")
             val = os.path.abspath(val)
+
+        if validate:
+            self._vol_install = None
+            self._vol_object = None
+            self._type = None
+
         self.__validate_wrapper("_path", val, validate)
     path = property(_get_path, _set_path)
 
@@ -608,7 +680,9 @@ class VirtualDisk(VirtualDevice):
             else:
                 dtype = self.TYPE_BLOCK
         elif self.path:
-            if _util.stat_disk(self.path)[0]:
+            if os.path.isdir(self.path):
+                dtype = self.TYPE_DIR
+            elif _util.stat_disk(self.path)[0]:
                 dtype = self.TYPE_FILE
             else:
                 dtype = self.TYPE_BLOCK
@@ -720,69 +794,85 @@ class VirtualDisk(VirtualDevice):
         """
         vol = None
         verr = None
+
+        def lookup_vol_by_path():
+            try:
+                return self.conn.storageVolLookupByPath(self.path)
+            except:
+                return None
+
         pool = _util.lookup_pool_by_path(self.conn,
                                          os.path.dirname(self.path))
-        if pool:
-            # Is pool running?
-            if pool.info()[0] != libvirt.VIR_STORAGE_POOL_RUNNING:
-                pool = None
+        vol = lookup_vol_by_path()
 
+
+        # Is pool running?
+        if pool and pool.info()[0] != libvirt.VIR_STORAGE_POOL_RUNNING:
+            pool = None
+
+        # Attempt to lookup path as a storage volume
+        try:
+            if vol:
+                vol.info()
+        except:
             try:
+                try:
+                    # Pool may need to be refreshed, but if it errors,
+                    # invalidate it
+                    if pool:
+                        pool.refresh(0)
+                except:
+                    pool = None
+                    raise
+
                 vol = self.conn.storageVolLookupByPath(self.path)
                 vol.info()
             except Exception, e:
                 vol = None
+                verr = str(e)
 
-                try:
-                    try:
-                        # Pool may need to be refreshed, but if it errors,
-                        # invalidate it
-                        pool.refresh(0)
-                    except:
-                        pool = None
-                        raise
-                    vol = self.conn.storageVolLookupByPath(self.path)
-                    vol.info()
-                except Exception, e:
-                    vol = None
-                    verr = str(e)
-
-        if not vol:
-            # Path wasn't a volume. See if base of path is a managed
-            # pool, and if so, setup a StorageVolume object
-            if pool:
-                if self.size == None:
-                    raise ValueError(_("Size must be specified for non "
-                                       "existent volume path '%s'" % \
-                                        self.path))
-                logging.debug("Path '%s' is target for pool '%s'. "
-                              "Creating volume '%s'." % \
-                              (os.path.dirname(self.path), pool.name(),
-                               os.path.basename(self.path)))
-                volclass = Storage.StorageVolume.get_volume_for_pool(pool_object=pool)
-                cap = (self.size * 1024 * 1024 * 1024)
-                if self.sparse:
-                    alloc = 0
-                else:
-                    alloc = cap
-                vol = volclass(name=os.path.basename(self.path),
-                               capacity=cap, allocation=alloc, pool=pool)
-                self._set_vol_install(vol, validate=False)
-            elif self._is_remote():
-
-                if not verr:
-                    # Since there is no error, no pool was ever found
-                    err = (_("Cannot use storage '%(path)s': '%(rootdir)s' is "
-                             "not managed on the remote host.") %
-                             { 'path' : self.path,
-                               'rootdir' : os.path.dirname(self.path)})
-                else:
-                    err = (_("Cannot use storage %(path)s: %(err)s") %
-                           { 'path' : self.path, 'err' : verr })
-
-                raise ValueError(err)
-        else:
+        if vol:
             self._set_vol_object(vol, validate=False)
+            return
+
+        if not pool:
+            if not self._is_remote():
+                # Building local disk
+                return
+
+            if not verr:
+                # Since there is no error, no pool was ever found
+                err = (_("Cannot use storage '%(path)s': '%(rootdir)s' is "
+                         "not managed on the remote host.") %
+                         { 'path' : self.path,
+                           'rootdir' : os.path.dirname(self.path)})
+            else:
+                err = (_("Cannot use storage %(path)s: %(err)s") %
+                        { 'path' : self.path, 'err' : verr })
+
+            raise ValueError(err)
+
+        # Path wasn't a volume. See if base of path is a managed
+        # pool, and if so, setup a StorageVolume object
+        if self.size == None:
+            raise ValueError(_("Size must be specified for non "
+                               "existent volume path '%s'" % self.path))
+
+        logging.debug("Path '%s' is target for pool '%s'. "
+                      "Creating volume '%s'." %
+                      (os.path.dirname(self.path), pool.name(),
+                       os.path.basename(self.path)))
+
+        volclass = Storage.StorageVolume.get_volume_for_pool(pool_object=pool)
+        cap = (self.size * 1024 * 1024 * 1024)
+        if self.sparse:
+            alloc = 0
+        else:
+            alloc = cap
+
+        vol = volclass(name=os.path.basename(self.path),
+                       capacity=cap, allocation=alloc, pool=pool)
+        self._set_vol_install(vol, validate=False)
 
 
     def __sync_params(self):
@@ -840,6 +930,8 @@ class VirtualDisk(VirtualDevice):
         function to validate all the complex interaction between the various
         disk parameters.
         """
+        if not self._validate:
+            return
 
         if self.__no_storage():
             # No storage specified for a removable device type (CDROM, floppy)
@@ -893,8 +985,9 @@ class VirtualDisk(VirtualDevice):
         if not create_media:
             # Make sure we have access to the local path
             if not managed_storage:
-                if os.path.isdir(self.path) and not _util.is_vdisk(self.path):
-                    # vdisk _is_ a directory.
+                if (os.path.isdir(self.path) and
+                    not _util.is_vdisk(self.path) and
+                    not self.device == self.DEVICE_FLOPPY):
                     raise ValueError(_("The path '%s' must be a file or a "
                                        "device, not a directory") % self.path)
 
@@ -986,7 +1079,7 @@ class VirtualDisk(VirtualDevice):
 
         try:
             try:
-                fd = os.open(self.path, os.O_WRONLY | os.O_CREAT)
+                fd = os.open(self.path, os.O_WRONLY | os.O_CREAT | os.O_DSYNC)
                 if self.sparse:
                     os.ftruncate(fd, size_bytes)
                     progresscb.update(self.size)
@@ -1059,15 +1152,22 @@ class VirtualDisk(VirtualDevice):
             if dst_fd is not None:
                 os.close(dst_fd)
 
-    def setup(self, progresscb=None):
+    def setup_dev(self, conn=None, meter=None):
         """
         Build storage (if required)
 
         If storage doesn't exist (a non-existent file 'path', or 'vol_install'
         was specified), we create it.
 
-        @param progresscb: progress meter
-        @type progresscb: instanceof urlgrabber.BaseMeter
+        @param conn: Optional connection to use if self.conn not specified
+        @param meter: Progress meter to report file creation on
+        @type meter: instanceof urlgrabber.BaseMeter
+        """
+        return self.setup(meter)
+
+    def setup(self, progresscb=None):
+        """
+        DEPRECATED: Please use setup_dev instead
         """
         if not progresscb:
             progresscb = progress.BaseMeter()
@@ -1093,7 +1193,7 @@ class VirtualDisk(VirtualDevice):
                          takes precedence.
         @type disknode: C{str}
         """
-        typeattr = 'file'
+        typeattr = self.type
         if self.type == VirtualDisk.TYPE_BLOCK:
             typeattr = 'dev'
 
@@ -1116,7 +1216,7 @@ class VirtualDisk(VirtualDevice):
         if not dname and self.driver_cache:
             self.driver_name = "qemu"
 
-        if not self.driver_name is None:
+        if path and not self.driver_name is None:
             dtypexml = ""
             if not self.driver_type is None:
                 dtypexml = " type='%s'" % self.driver_type
@@ -1199,9 +1299,6 @@ class VirtualDisk(VirtualDevice):
                  return_names passed)
         @rtype: C{bool}
         """
-        active, inactive = _util.fetch_all_guests(conn)
-        vms = active + inactive
-
         if self.vol_object:
             path = self.vol_object.path()
         else:
@@ -1210,24 +1307,15 @@ class VirtualDisk(VirtualDevice):
         if not path:
             return False
 
-        def count_cb(ctx):
-            c = 0
+        if not conn:
+            conn = self.conn
 
-            c += ctx.xpathEval("count(/domain/devices/disk/source[@dev='%s'])" % path)
-            c += ctx.xpathEval("count(/domain/devices/disk/source[@file='%s'])" % path)
-            return c
-
-        count = 0
-        names = []
-        for vm in vms:
-            xml = vm.XMLDesc(0)
-            tmpcount = _util.get_xml_path(xml, func = count_cb)
-            if tmpcount:
-                count += tmpcount
-                names.append(vm.name())
+        check_conflict = self.shareable
+        names = self.path_in_use_by(conn, path,
+                                    check_conflict=check_conflict)
 
         ret = False
-        if count > 0:
+        if names:
             ret = True
         if return_names:
             ret = names
@@ -1347,12 +1435,11 @@ class VirtualDisk(VirtualDevice):
         if prefix is None:
             raise ValueError(_("Cannot determine device bus/type."))
 
-        # Special case: IDE cdrom must be hdc
+        # Special case: IDE cdrom should prefer hdc for back compat
         if self.device == self.DEVICE_CDROM and prefix == "hd":
             if "hdc" not in skip_targets:
                 self.target = "hdc"
                 return self.target
-            raise ValueError(_("IDE CDROM must use 'hdc', but target in use."))
 
         # Regular scanning
         for i in range(maxnode):
