@@ -20,7 +20,9 @@
 import VirtualDevice
 import NodeDeviceParser
 import logging
+import libvirt
 
+from virtinst import support
 from virtinst import _util
 from virtinst import _virtinst as _
 
@@ -30,11 +32,15 @@ class VirtualHostDevice(VirtualDevice.VirtualDevice):
 
     def device_from_node(conn, name=None, nodedev=None):
         """
-        Convert the passed libvirt node device name to a VirtualHostDevice
-        instance, with proper error reporting.
+        Convert the passed device name to a VirtualHostDevice
+        instance, with proper error reporting. Name can be any of the
+        values accepted by NodeDeviceParser.lookupNodeName. If a node
+        device name is not specified, a virtinst.NodeDevice instance can
+        be passed in to create a dev from.
 
         @param conn: libvirt.virConnect instance to perform the lookup on
-        @param name: libvirt node device name to lookup
+        @param name: optional libvirt node device name to lookup
+        @param nodedev: optional L{virtinst.NodeDevice} instance to use
 
         @rtype: L{virtinst.VirtualHostDevice} instance
         """
@@ -147,8 +153,13 @@ class VirtualHostDeviceUSB(VirtualHostDevice):
                                  " 'device' are required."))
         return xml
 
+    def setup_dev(self, conn=None, meter=None):
+        return self.setup(conn)
 
     def setup(self, conn = None):
+        """
+        DEPRECATED: Please use setup_dev instead
+        """
         if not conn:
             conn = self.conn
 
@@ -191,13 +202,19 @@ class VirtualHostDevicePCI(VirtualHostDevice):
         xml = "        <address domain='%s' bus='%s' slot='%s' function='%s'/>\n"
         return xml % (self.domain, self.bus, self.slot, self.function)
 
-    def setup(self, conn = None):
+    def setup_dev(self, conn=None, meter=None):
         """
         Perform DeviceDetach and DeviceReset calls if necessary
 
         @param conn: libvirt virConnect instance to use (defaults to devices
                      connection)
         """
+        return self.setup(conn)
+
+    def setup(self, conn = None):
+        """
+        DEPRECATED: Please use setup_dev instead
+        """
         if not conn:
             conn = self.conn
 
@@ -205,12 +222,16 @@ class VirtualHostDevicePCI(VirtualHostDevice):
             return
 
         try:
-            # Do this as a sanity check, so that we don't fail at domain
-            # start time. This is independent of the 'managed' state, since
-            # this should work regardless.
-            node = conn.nodeDeviceLookupByName(self._nodedev.name)
-            node.dettach()
-            node.reset()
+            try:
+                # Do this as a sanity check, so that we don't fail at domain
+                # start time. This is independent of the 'managed' state, since
+                # this should work regardless.
+                node = conn.nodeDeviceLookupByName(self._nodedev.name)
+                node.dettach()
+                node.reset()
+            except libvirt.libvirtError, e:
+                if not support.is_error_nosupport(e):
+                    raise
         except Exception, e:
             raise RuntimeError(_("Could not detach PCI device: %s" % str(e)))
 
