@@ -17,10 +17,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import re
 import logging
 import libvirt
-import __builtin__
 
 import _util
 import VirtualDevice
@@ -82,17 +80,7 @@ class VirtualNetworkInterface(VirtualDevice.VirtualDevice):
     def get_macaddr(self):
         return self._macaddr
     def set_macaddr(self, val):
-        if val is None:
-            self._macaddr = None
-            return
-
-        if __builtin__.type(val) is not str:
-            raise ValueError, _("MAC address must be a string.")
-
-        form = re.match("^([0-9a-fA-F]{1,2}:){5}[0-9a-fA-F]{1,2}$", val)
-        if form is None:
-            raise ValueError(_("MAC address must be of the format "
-                               "AA:BB:CC:DD:EE:FF"))
+        _util.validate_macaddr(val)
         self._macaddr = val
     macaddr = property(get_macaddr, set_macaddr)
 
@@ -144,7 +132,16 @@ class VirtualNetworkInterface(VirtualDevice.VirtualDevice):
             return (False, _("The MAC address you entered is already in use by another inactive virtual machine."))
         return (False, None)
 
-    def setup(self, conn):
+    def setup_dev(self, conn=None, meter=None):
+        return self.setup(conn)
+
+    def setup(self, conn=None):
+        """
+        DEPRECATED: Please use setup_dev instead
+        """
+        if not conn:
+            conn = self.conn
+
         if self.macaddr is None:
             while 1:
                 self.macaddr = _util.randomMAC(type=conn.getType().lower())
