@@ -28,6 +28,7 @@ from version import __version__
 __version_info__ = tuple([ int(num) for num in __version__.split('.')])
 
 import Storage
+import Interface
 from Guest import Guest, XenGuest
 from VirtualDevice import VirtualDevice
 from VirtualNetworkInterface import VirtualNetworkInterface, \
@@ -40,15 +41,19 @@ from VirtualHostDevice import (VirtualHostDevice, VirtualHostDeviceUSB,
                                VirtualHostDevicePCI)
 from VirtualCharDevice import VirtualCharDevice
 from VirtualVideoDevice import VirtualVideoDevice
+from VirtualController import VirtualController
+from VirtualWatchdog import VirtualWatchdog
 from FullVirtGuest import FullVirtGuest
 from ParaVirtGuest import ParaVirtGuest
 from DistroInstaller import DistroInstaller
 from PXEInstaller import PXEInstaller
 from LiveCDInstaller import LiveCDInstaller
 from ImportInstaller import ImportInstaller
-from ImageManager import ImageInstaller
+from ImageInstaller import ImageInstaller
 from CloneManager import CloneDesign
 from User import User
+from Clock import Clock
+from Seclabel import Seclabel
 import util
 import support
 
@@ -58,7 +63,9 @@ __all__ = ["Guest", "XenGuest", "VirtualNetworkInterface",
            "XenNetworkInterface", "VirtualGraphics", "VirtualAudio",
            "VirtualDisk", "XenDisk", "FullVirtGuest", "ParaVirtGuest",
            "DistroInstaller", "PXEInstaller", "LiveCDInstaller",
-           "ImportInstaller", "ImageInstaller", "CloneDesign", "Storage",
-           "User", "util", "support", "VirtualDevice",
+           "ImportInstaller", "ImageInstaller", "CloneDesign",
+           "Storage", "Interface",
+           "User", "util", "support", "VirtualDevice", "Clock", "Seclabel",
            "VirtualHostDevice", "VirtualHostDeviceUSB", "VirtualVideoDevice",
-           "VirtualHostDevicePCI", "VirtualCharDevice", "VirtualInputDevice"]
+           "VirtualHostDevicePCI", "VirtualCharDevice", "VirtualInputDevice",
+           "VirtualController", "VirtualWatchdog"]
