@@ -22,7 +22,7 @@
 import os, sys
 import logging
 import logging.handlers
-import locale
+import gettext, locale
 import optparse
 from optparse import OptionValueError, OptionParser
 
@@ -30,7 +30,8 @@ import libvirt
 import _util
 import virtinst
 from virtinst import CapabilitiesParser, VirtualNetworkInterface, \
-                     VirtualGraphics, VirtualAudio, VirtualDisk, User
+                     VirtualGraphics, VirtualAudio, VirtualDisk, User, \
+                     VirtualVideoDevice
 from virtinst import _virtinst as _
 
 MIN_RAM = 64
@@ -44,6 +45,29 @@ def check_if_test_uri_remote(uri):
         _util.is_uri_remote = lambda uri_: True
     return uri
 
+class VirtStreamHandler(logging.StreamHandler):
+
+    def emit(self, record):
+        """
+        Based on the StreamHandler code from python 2.6: ripping out all
+        the unicode handling and just uncoditionally logging seems to fix
+        logging backtraces with unicode locales (for me at least).
+
+        No doubt this is atrocious, but it WORKSFORME!
+        """
+        try:
+            msg = self.format(record)
+            stream = self.stream
+            fs = "%s\n"
+
+            stream.write(fs % msg)
+
+            self.flush()
+        except (KeyboardInterrupt, SystemExit):
+            raise
+        except:
+            self.handleError(record)
+
 class VirtOptionParser(OptionParser):
     '''Subclass to get print_help to work properly with non-ascii text'''
 
@@ -56,13 +80,25 @@ class VirtOptionParser(OptionParser):
     def print_help(self, file=None):
         if file is None:
             file = sys.stdout
+
         encoding = self._get_encoding(file)
-        file.write(self.format_help().encode(encoding, "replace"))
+        helpstr = self.format_help()
+        try:
+            encodedhelp = helpstr.encode(encoding, "replace")
+        except UnicodeError:
+            # I don't know why the above fails hard, unicode makes my head
+            # spin. Just printing the format_help() output seems to work
+            # quite fine, with the occasional character ?.
+            encodedhelp = helpstr
+
+        file.write(encodedhelp)
 
 class VirtHelpFormatter(optparse.IndentedHelpFormatter):
     """
     Subclass the default help formatter to allow printing newline characters
     in --help output. The way we do this is a huge hack :(
+
+    Inspiration: http://groups.google.com/group/comp.lang.python/browse_thread/thread/6df6e6b541a15bc2/09f28e26af0699b1
     """
     oldwrap = None
 
@@ -80,18 +116,37 @@ class VirtHelpFormatter(optparse.IndentedHelpFormatter):
         ret = []
         for line in text.split("\n"):
             ret.extend(self.oldwrap(line, width))
-        if ret:
-            print "\n".join(ret)
         return ret
 #
 # Setup helpers
 #
 
+def setupParser(usage=None):
+    parse_class = VirtOptionParser
+
+    parser = parse_class(usage=usage,
+                         formatter=VirtHelpFormatter(),
+                         version=virtinst.__version__)
+    return parser
+
+def setupGettext():
+    locale.setlocale(locale.LC_ALL, '')
+    gettext.bindtextdomain(virtinst.gettext_app, virtinst.gettext_dir)
+    gettext.install(virtinst.gettext_app, virtinst.gettext_dir)
+
 def setupLogging(appname, debug=False):
     # set up logging
     vi_dir = os.path.expanduser("~/.virtinst")
-    if not os.access(vi_dir,os.W_OK):
-        os.mkdir(vi_dir, 0751)
+    if not os.access(vi_dir, os.W_OK):
+        if os.path.exists(vi_dir):
+            raise RuntimeError("No write access to directory %s" % vi_dir)
+
+        try:
+            os.mkdir(vi_dir, 0751)
+        except IOError, e:
+            raise RuntimeError("Could not create directory %s: %s" %
+                               (vi_dir, e))
+
 
     dateFormat = "%a, %d %b %Y %H:%M:%S"
     fileFormat = "[%(asctime)s " + appname + " %(process)d] %(levelname)s (%(module)s:%(lineno)d) %(message)s"
@@ -108,7 +163,7 @@ def setupLogging(appname, debug=False):
                                                dateFormat))
     rootLogger.addHandler(fileHandler)
 
-    streamHandler = logging.StreamHandler(sys.stderr)
+    streamHandler = VirtStreamHandler(sys.stderr)
     if debug:
         streamHandler.setLevel(logging.DEBUG)
         streamHandler.setFormatter(logging.Formatter(streamDebugFormat,
@@ -314,7 +369,11 @@ def prompt_for_yes_or_no(warning, question):
     errmsg = warning + _(" (Use --prompt or --force to override)")
 
     while 1:
-        inp = prompt_for_input(errmsg, warning + question, None)
+        msg = warning
+        if question:
+            msg += ("\n" + question)
+
+        inp = prompt_for_input(errmsg, msg, None)
         try:
             res = yes_or_no(inp)
             break
@@ -365,7 +424,8 @@ def graphics_option_group(parser):
 # Specific function for disk prompting. Returns a validated VirtualDisk
 # device.
 #
-def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True):
+def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True,
+                path_to_clone=None):
 
     retry_path = True
     conn = arg_dict.get("conn")
@@ -383,6 +443,10 @@ def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True):
 
         msg = None
         patherr = _("A disk path must be specified.")
+        if path_to_clone:
+            patherr = (_("A disk path must be specified to clone '%s'.") %
+                       path_to_clone)
+
         if not prompt_txt:
             msg = _("What would you like to use as the disk (file path)?")
             if not size is None:
@@ -673,13 +737,21 @@ def digest_networks(conn, macs, bridges, networks, nics = 0):
 
     return net_init_dicts
 
-def get_graphics(vnc, vncport, vnclisten, nographics, sdl, keymap, guest):
+def get_graphics(vnc, vncport, vnclisten, nographics, sdl, keymap,
+                 video_models, guest):
+    video_models = video_models or []
+
     if ((vnc and nographics) or
         (vnc and sdl) or
         (sdl and nographics)):
         raise ValueError, _("Can't specify more than one of VNC, SDL, "
                             "or --nographics")
 
+    for model in video_models:
+        dev = virtinst.VirtualVideoDevice(guest.conn)
+        dev.model_type = model
+        guest.add_device(dev)
+
     if not (vnc or nographics or sdl):
         if "DISPLAY" in os.environ.keys():
             logging.debug("DISPLAY is set: graphics defaulting to VNC.")
@@ -692,6 +764,11 @@ def get_graphics(vnc, vncport, vnclisten, nographics, sdl, keymap, guest):
         guest.graphics_dev = None
         return
 
+    # After this point, we are using graphics, so add a video device
+    # if one wasn't passed
+    if not video_models:
+        guest.add_device(VirtualVideoDevice(conn=guest.conn))
+
     if sdl is not None:
         guest.graphics_dev = VirtualGraphics(type=VirtualGraphics.TYPE_SDL)
         return
@@ -717,13 +794,18 @@ def get_graphics(vnc, vncport, vnclisten, nographics, sdl, keymap, guest):
 
         guest.graphics_dev.keymap = use_keymap
 
-def get_sound(sound, guest):
+def get_sound(old_sound_bool, sound_opts, guest):
+    if not sound_opts:
+        if old_sound_bool:
+            # Use os default
+            guest.sound_devs.append(VirtualAudio(conn=guest.conn))
+        return
+
+    for model in listify(sound_opts):
+        dev = VirtualAudio(conn=guest.conn)
+        dev.model = model
+        guest.add_device(dev)
 
-    # Sound is just a boolean value, so just specify a default of 'es1370'
-    # model since this should provide audio out of the box for most modern
-    # distros
-    if sound:
-        guest.sound_devs.append(VirtualAudio(model="es1370"))
 
 def get_hostdevs(hostdevs, guest):
     if not hostdevs:
@@ -734,15 +816,6 @@ def get_hostdevs(hostdevs, guest):
                                                           name=devname)
         guest.hostdevs.append(dev)
 
-def get_video(video_models, guest):
-    if not video_models:
-        return
-
-    for model in video_models:
-        dev = virtinst.VirtualVideoDevice(guest.conn)
-        dev.model_type = model
-        guest.add_device(dev)
-
 ### Option parsing
 def check_before_store(option, opt_str, value, parser):
     if len(value) == 0:
