@@ -19,46 +19,95 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import libvirt
-
-import _util
+import support
+from VirtualDevice import VirtualDevice
 from virtinst import _virtinst as _
 
+HV_ALL = "all"
+
 """
 Default values for OS_TYPES keys. Can be overwritten at os_type or
 variant level
 """
-DEFAULTS = { \
-    "acpi": True,
-    "apic": True,
-    "clock": "utc",
-    "continue": False,
-    "distro": None,
-    "label": None,
+
+NET   = VirtualDevice.VIRTUAL_DEV_NET
+DISK  = VirtualDevice.VIRTUAL_DEV_DISK
+INPUT = VirtualDevice.VIRTUAL_DEV_INPUT
+SOUND = VirtualDevice.VIRTUAL_DEV_AUDIO
+VIDEO = VirtualDevice.VIRTUAL_DEV_VIDEO
+
+VIRTIO_DISK = {
+    "bus" : [
+        (support.SUPPORT_CONN_HV_VIRTIO, "virtio"),
+    ]
+}
+
+VIRTIO_NET = {
+    "model" : [
+        (support.SUPPORT_CONN_HV_VIRTIO, "virtio"),
+    ]
+}
+
+USB_TABLET = {
+    "type" : [
+        (HV_ALL, "tablet"),
+    ],
+    "bus"  : [
+        (HV_ALL, "usb"),
+    ]
+}
+
+VGA_VIDEO = {
+    "model_type": [
+        (HV_ALL, "vga"),
+    ]
+}
+
+DEFAULTS = {
+    "acpi":             True,
+    "apic":             True,
+    "clock":            "utc",
+    "continue":         False,
+    "distro":           None,
+    "label":            None,
     "pv_cdrom_install": False,
+
     "devices" : {
-     #  "devname" : { "attribute" : [( ["applicable", "hv-type", list"],
-     #                               "recommended value for hv-types" ),]},
-        "input"   : {
+        #  "devname" : { "attribute" : [( ["applicable", "hv-type", list"],
+        #                               "recommended value for hv-types" ),]},
+        INPUT   : {
             "type" : [
-                (["all"], "mouse")
+                (HV_ALL, "mouse")
             ],
             "bus"  : [
-                (["all"], "ps2")
+                (HV_ALL, "ps2")
             ],
         },
 
-        "disk"    : {
+        DISK    : {
             "bus"  : [
-                (["all"], None)
+                (HV_ALL, None)
             ],
         },
 
-        "net"     : {
+        NET     : {
             "model": [
-                (["all"], None)
+                (HV_ALL, None)
             ],
         },
+
+        SOUND : {
+            "model": [
+                (support.SUPPORT_CONN_HV_SOUND_AC97, "ac97"),
+                (HV_ALL, "es1370"),
+            ]
+        },
+
+        VIDEO : {
+            "model_type": [
+                (HV_ALL, "cirrus"),
+            ]
+        },
     }
 }
 
@@ -84,92 +133,66 @@ def sort_helper(tosort):
 
     return retlist
 
-def parse_key_entry(conn, hv_type, key_entry):
-    d = _util.get_uri_driver(conn.getURI())
-    libver = libvirt.getVersion()
-    try:
-        drvver = libvirt.getVersion(d)[1]
-    except:
-        drvver = 0
-
+def parse_key_entry(conn, hv_type, key_entry, defaults):
     ret = None
+    found = False
     if type(key_entry) == list:
-        # List of tuples with hv_type, version, etc. mappings
+
+        # List of tuples with (support -> value) mappings
         for tup in key_entry:
-            exp_hvs = tup[0]
-            if type(exp_hvs) != list:
-                exp_hvs = [exp_hvs]
-            exp_hv_ver = 0
-            exp_lib_ver = 0
-            val = tup[-1]
 
-            if len(tup) > 2:
-                exp_hv_ver = tup[1]
-            if len(tup) > 3:
-                exp_lib_ver = tup[2]
+            support_key = tup[0]
+            value = tup[1]
 
-            if hv_type not in exp_hvs and "all" not in exp_hvs:
-                continue
+            # HV_ALL means don't check for support, just return the value
+            if support_key != HV_ALL:
+                support_ret = support.check_conn_hv_support(conn,
+                                                            support_key,
+                                                            hv_type)
 
-            if exp_hv_ver and drvver > exp_hv_ver:
-                continue
+                if support_ret != True:
+                    continue
 
-            if exp_lib_ver and libver > exp_lib_ver:
-                continue
-
-            ret = val
+            found = True
+            ret = value
             break
     else:
+        found = True
         ret = key_entry
 
+    if not found and defaults:
+        ret = parse_key_entry(conn, hv_type, defaults, None)
+
     return ret
 
 def lookup_osdict_key(conn, hv_type, os_type, var, key):
 
-    dictval = DEFAULTS[key]
+    defaults = DEFAULTS[key]
+    dictval = defaults
     if os_type:
         if var and OS_TYPES[os_type]["variants"][var].has_key(key):
             dictval = OS_TYPES[os_type]["variants"][var][key]
         elif OS_TYPES[os_type].has_key(key):
             dictval = OS_TYPES[os_type][key]
 
-    return parse_key_entry(conn, hv_type, dictval)
+    return parse_key_entry(conn, hv_type, dictval, defaults)
 
 
 def lookup_device_param(conn, hv_type, os_type, var, device_key, param):
 
     os_devs = lookup_osdict_key(conn, hv_type, os_type, var, "devices")
-    default_devs = DEFAULTS["devices"]
+    defaults = DEFAULTS["devices"]
 
-    for devs in [os_devs, default_devs]:
+    for devs in [os_devs, defaults]:
         if not devs.has_key(device_key):
             continue
 
-        return parse_key_entry(conn, hv_type, devs[device_key][param])
+        return parse_key_entry(conn, hv_type, devs[device_key][param],
+                               defaults.get(param))
 
     raise RuntimeError(_("Invalid dictionary entry for device '%s %s'" %
                        (device_key, param)))
 
-VIRTIO_DISK = {
-    "bus" : [
-        (["kvm"], "virtio"),
-    ]
-}
-
-VIRTIO_NET = {
-    "model" : [
-        (["kvm"], "virtio"),
-    ]
-}
-
-USB_TABLET = {
-    "type" : [
-        (["all"], "tablet"),
-    ],
-    "bus"  : [
-        (["all"], "usb"),
-    ]
-}
 
 # NOTE: keep variant keys using only lowercase so we can do case
 #       insensitive checks on user passed input
@@ -188,14 +211,14 @@ OS_TYPES = {
         "rhel5.4": { "label": "Red Hat Enterprise Linux 5.4 or later",
                      "distro": "rhel",
                       "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
                       },},
         "rhel6": { "label": "Red Hat Enterprise Linux 6", "distro": "rhel",
                    "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
-                        "input": USB_TABLET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
+                        INPUT: USB_TABLET,
                    }},
         "fedora5": { "sortby": "fedora05",
                      "label": "Fedora Core 5", "distro": "fedora" },
@@ -211,82 +234,103 @@ OS_TYPES = {
                         # Apparently F9 has selinux errors when installing
                         # with virtio:
                         # https://bugzilla.redhat.com/show_bug.cgi?id=470386
-                        #"disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
+                        #DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
                       }},
         "fedora10": { "label": "Fedora 10", "distro": "fedora",
                       "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
                       }},
         "fedora11": { "label": "Fedora 11", "distro": "fedora",
                       "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
-                        "input": USB_TABLET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
+                        INPUT: USB_TABLET,
                      }},
         "fedora12": { "label": "Fedora 12", "distro": "fedora",
                       "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
-                        "input": USB_TABLET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
+                        INPUT: USB_TABLET,
                      }},
         "fedora13": { "label": "Fedora 13", "distro": "fedora",
                       "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
-                        "input": USB_TABLET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
+                        INPUT: USB_TABLET,
                      }},
+
         "sles10": { "label": "Suse Linux Enterprise Server",
                     "distro": "suse" },
         "sles11": { "label": "Suse Linux Enterprise Server 11",
                     "distro": "suse",
                       "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
                       },
                   },
+
+        "mandriva2009": { "label": "Mandriva Linux 2009 and earlier",
+                          "distro": "mandriva" },
+        "mandriva2010": { "label": "Mandriva Linux 2010 and later",
+                          "distro": "mandriva",
+                          "devices" : {
+                            DISK : VIRTIO_DISK,
+                            NET  : VIRTIO_NET,
+                        },
+                  },
+
+        "mes5": { "label": "Mandriva Enterprise Server 5.0",
+                    "distro": "mandriva" },
+        "mes5.1": { "label": "Mandriva Enterprise Server 5.1 and later",
+                    "distro": "mandriva",
+                    "devices" : {
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
+                    },
+                  },
         "debianetch": { "label": "Debian Etch", "distro": "debian" },
         "debianlenny": { "label": "Debian Lenny", "distro": "debian",
                       "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
                       }},
         "debiansqueeze": { "label": "Debian Squeeze", "distro": "debian",
                       "devices" : {
-                        "disk" : VIRTIO_DISK,
-                        "net"  : VIRTIO_NET,
-                        "input": USB_TABLET,
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
+                        INPUT: USB_TABLET,
                      }},
         "ubuntuhardy": { "label": "Ubuntu 8.04 LTS (Hardy Heron)",
                          "distro": "ubuntu",
                          "devices" : {
-                            "net"  : VIRTIO_NET,
+                            NET  : VIRTIO_NET,
                          }},
         "ubuntuintrepid": { "label": "Ubuntu 8.10 (Intrepid Ibex)",
                             "distro": "ubuntu",
                             "devices" : {
-                              "net"  : VIRTIO_NET,
+                              NET  : VIRTIO_NET,
                            }},
         "ubuntujaunty": { "label": "Ubuntu 9.04 (Jaunty Jackalope)",
                           "distro": "ubuntu",
                           "devices" : {
-                            "disk" : VIRTIO_DISK,
-                            "net"  : VIRTIO_NET,
+                            DISK : VIRTIO_DISK,
+                            NET  : VIRTIO_NET,
                         }},
         "ubuntukarmic": { "label": "Ubuntu 9.10 (Karmic Koala)",
                           "distro": "ubuntu",
                           "devices" : {
-                            "disk" : VIRTIO_DISK,
-                            "net"  : VIRTIO_NET,
+                            DISK : VIRTIO_DISK,
+                            NET  : VIRTIO_NET,
                         }},
         "generic24": { "label": "Generic 2.4.x kernel" },
         "generic26": { "label": "Generic 2.6.x kernel" },
         "virtio26": { "sortby": "genericvirtio26",
                       "label": "Generic 2.6.25 or later kernel with virtio",
                       "devices" : {
-                            "disk" : VIRTIO_DISK,
-                            "net"  : VIRTIO_NET,
+                            DISK : VIRTIO_DISK,
+                            NET  : VIRTIO_NET,
                     }},
 
     },
@@ -297,20 +341,27 @@ OS_TYPES = {
     "clock": "localtime",
     "continue": True,
     "devices" : {
-        "input" : USB_TABLET,
+        INPUT : USB_TABLET,
+        VIDEO : VGA_VIDEO,
     },
     "variants": {
         "winxp":{ "label": "Microsoft Windows XP (x86)",
-                  "acpi": [ ("xen", 3001000, False),
-                            ("all", True ), ],
-                  "apic": [ ("xen", 3001000, False),
-                            ("all", True ), ], },
+                  "acpi": [
+                    (support.SUPPORT_CONN_HV_SKIP_DEFAULT_ACPI, False),
+                  ],
+                  "apic": [
+                    (support.SUPPORT_CONN_HV_SKIP_DEFAULT_ACPI, False),
+                  ],
+        },
         "winxp64":{ "label": "Microsoft Windows XP (x86_64)" },
         "win2k": { "label": "Microsoft Windows 2000",
-                  "acpi": [ ("xen", 3001000, False),
-                            ("all", True ), ],
-                  "apic": [ ("xen", 3001000, False),
-                            ("all", True ), ], },
+                  "acpi": [
+                    (support.SUPPORT_CONN_HV_SKIP_DEFAULT_ACPI, False),
+                  ],
+                  "apic": [
+                    (support.SUPPORT_CONN_HV_SKIP_DEFAULT_ACPI, False),
+                  ],
+        },
         "win2k3": { "label": "Microsoft Windows 2003" },
         "win2k8": { "label": "Microsoft Windows 2008" },
         "vista": { "label": "Microsoft Windows Vista" },
@@ -326,12 +377,12 @@ OS_TYPES = {
         "solaris9": { "label": "Sun Solaris 9", },
         "solaris10": { "label": "Sun Solaris 10",
                        "devices" : {
-                        "input" : USB_TABLET,
+                            INPUT : USB_TABLET,
                          },
                        },
         "opensolaris": { "label": "Sun OpenSolaris",
                        "devices" : {
-                           "input" : USB_TABLET,
+                            INPUT : USB_TABLET,
                          },
                        },
     },
@@ -343,17 +394,17 @@ OS_TYPES = {
         "freebsd6": { "label": "Free BSD 6.x" ,
                       # http://www.nabble.com/Re%3A-Qemu%3A-bridging-on-FreeBSD-7.0-STABLE-p15919603.html
                       "devices" : {
-                        "net" : { "model" : [ (["all"], "ne2k_pci") ] }
+                        NET : { "model" : [ (HV_ALL, "ne2k_pci") ] }
                       }},
         "freebsd7": { "label": "Free BSD 7.x" ,
                       "devices" : {
-                        "net" : { "model" : [ (["all"], "ne2k_pci") ] }
+                        NET : { "model" : [ (HV_ALL, "ne2k_pci") ] }
                       }},
         "openbsd4": { "label": "Open BSD 4.x" ,
                       # http://calamari.reverse-dns.net:980/cgi-bin/moin.cgi/OpenbsdOnQemu
                       # https://www.redhat.com/archives/et-mgmt-tools/2008-June/msg00018.html
                       "devices" : {
-                        "net"  : { "model" : [ (["all"], "pcnet") ] }
+                        NET  : { "model" : [ (HV_ALL, "pcnet") ] }
                     }},
     },
 },
