@@ -31,12 +31,13 @@ SUPPORT_CONN_NODEDEV = 2
 SUPPORT_CONN_KEYMAP_AUTODETECT = 3
 SUPPORT_CONN_GETHOSTNAME = 4
 SUPPORT_CONN_DOMAIN_VIDEO = 5
-SUPPORT_CONN_DOMAIN_SOUND_AC97 = 6
 SUPPORT_CONN_NETWORK = 7
 SUPPORT_CONN_INTERFACE = 8
 
 # Flags for check_domain_support
 SUPPORT_DOMAIN_GETVCPUS = 1000
+SUPPORT_DOMAIN_XML_SECURE = 1001
+SUPPORT_DOMAIN_XML_INACTIVE = 1002
 
 # Flags for check_pool_support
 SUPPORT_STORAGE_CREATEVOLFROM = 2000
@@ -44,6 +45,13 @@ SUPPORT_STORAGE_CREATEVOLFROM = 2000
 # Flags for check_nodedev_support
 SUPPORT_NODEDEV_PCI_DETACH = 3000
 
+# Flags for check_interface_support
+SUPPORT_INTERFACE_XML_INACTIVE = 4000
+
+# Flags for check_conn_hv_support
+SUPPORT_CONN_HV_VIRTIO = 5000
+SUPPORT_CONN_HV_SKIP_DEFAULT_ACPI = 5001
+SUPPORT_CONN_HV_SOUND_AC97 = 5002
 
 """
 Possible keys:
@@ -62,11 +70,17 @@ Possible keys:
   "function" : Function name to check exists. If object not specified,
                function is checked against libvirt module.
   "args": Argument tuple to actually test object.function with.
+  "flag": A flag to check exists. This will be appended to the argument
+          list if args are provided, otherwise we will only check against
+          the local libvirt version.
 
   "drv_version" : A list of tuples of the form
                   (driver name (e.g qemu, xen, lxc), minimum supported version)
                  If a hypervisor is not listed, it is assumed to be NOT
                  SUPPORTED.
+  "hv_version" : A list of tuples of the same form as drv_version, however
+                 listing the actual <domain type='%s'/> from the XML.
+                 example: 'kvm'
 
 """
 
@@ -98,11 +112,6 @@ _support_dict = {
         "version" : 6005,
     },
 
-    SUPPORT_CONN_DOMAIN_SOUND_AC97 : {
-        "version" : 6000,
-        "force_version" : True,
-        "drv_version" : [ ("qemu", 11000) ],
-    },
 
     SUPPORT_CONN_NETWORK : {
         "function" : "virConnect.listNetworks",
@@ -121,6 +130,18 @@ _support_dict = {
         "args" : (),
     },
 
+    SUPPORT_DOMAIN_XML_INACTIVE : {
+        "function" : "virDomain.XMLDesc",
+        "args" : (),
+        "flag" : "VIR_DOMAIN_XML_INACTIVE",
+    },
+
+    SUPPORT_DOMAIN_XML_SECURE : {
+        "function" : "virDomain.XMLDesc",
+        "args" : (),
+        "flag" : "VIR_DOMAIN_XML_SECURE",
+    },
+
     # Pool checks
     # This can't ever require a pool object for back compat reasons
     SUPPORT_STORAGE_CREATEVOLFROM : {
@@ -134,6 +155,29 @@ _support_dict = {
         "function" : "virNodeDevice.dettach",
         "version" : 6001,
     },
+
+    # Interface checks
+    SUPPORT_INTERFACE_XML_INACTIVE : {
+        "function" : "virInterface.XMLDesc",
+        "args" : (),
+        "flag" : "VIR_INTERFACE_XML_INACTIVE",
+    },
+
+    # Conn HV checks
+    SUPPORT_CONN_HV_VIRTIO : {
+        "drv_version": [ ("qemu", 0) ],
+        "hv_version" : [ ("kvm", 0) ],
+    },
+
+    SUPPORT_CONN_HV_SKIP_DEFAULT_ACPI : {
+        "drv_version" : [ ("xen", -3001000)],
+    },
+
+    SUPPORT_CONN_HV_SOUND_AC97 : {
+        "version" : 6000,
+        "force_version" : True,
+        "drv_version" : [ ("qemu", 11000), ],
+    },
 }
 
 # Pull a connection object from the passed libvirt object
@@ -162,9 +206,13 @@ def _get_command(funcname, objname=None, obj=None):
 def _has_command(funcname, objname=None, obj=None):
     return bool(_get_command(funcname, objname, obj))
 
+# Make sure libvirt object has flag 'flag_name'
+def _get_flag(flag_name):
+    return _get_command(flag_name)
+
 # Try to call the passed function, and look for signs that libvirt or driver
 # doesn't support it
-def _try_command(func, args):
+def _try_command(func, args, check_all_error=False):
     try:
         func(*args)
 
@@ -172,8 +220,12 @@ def _try_command(func, args):
         if is_error_nosupport(e):
             return False
 
+        if check_all_error:
+            return False
+
     except Exception:
-        return True
+        # Other python exceptions likely mean the bindings are horked
+        return False
 
     return True
 
@@ -202,14 +254,25 @@ def _daemon_lib_ver(conn, force_version):
 
 # Return the hypervisor version
 def _hv_ver(conn):
+    drv_type = _util.get_uri_driver(conn.getURI())
+    args = ()
+
     cmd = _get_command("getVersion", obj=conn)
+    if not cmd:
+        cmd = _get_command("getVersion")
+        args = (drv_type,)
+
     if not cmd:
         return 0
 
-    if not _try_command(cmd, ()):
+    if not _try_command(cmd, args):
         return 0
 
-    return cmd()
+    ret = cmd(*args)
+    if type(ret) == tuple:
+        ret = ret[1]
+
+    return ret
 
 def _split_function_name(function):
     if not function:
@@ -221,7 +284,7 @@ def _split_function_name(function):
     else:
         return (output[0], output[1])
 
-def _check_support(conn, feature, obj=None):
+def _check_support(conn, feature, data=None):
     """
     Attempt to determine if a specific libvirt feature is support given
     the passed connection.
@@ -230,8 +293,9 @@ def _check_support(conn, feature, obj=None):
     @type  conn: virConnect
     @param feature: Feature type to check support for
     @type  feature: One of the SUPPORT_* flags
-    @param obj: Option libvirt object to use in feature checking
-    @type  obj: Could be virDomain, virNetwork, virStoragePool, etc..
+    @param data: Option libvirt object to use in feature checking
+    @type  data: Could be virDomain, virNetwork, virStoragePool,
+                hv name, etc
 
     @returns: True if feature is supported, False otherwise
     """
@@ -250,8 +314,10 @@ def _check_support(conn, feature, obj=None):
     minimum_libvirt_version = get_value("version") or 0
     force_version = get_value("force_version") or False
     drv_version = get_value("drv_version") or []
+    hv_version = get_value("hv_version") or []
     object_name, function_name = _split_function_name(get_value("function"))
     args = get_value("args")
+    flag = get_value("flag")
 
     actual_lib_ver = _local_lib_ver()
     actual_daemon_ver = _daemon_lib_ver(conn, force_version)
@@ -265,9 +331,17 @@ def _check_support(conn, feature, obj=None):
     if function_name:
         # Make sure function is present in either libvirt module or
         # object_name class
+        flag_tuple = ()
+
         if not _has_command(function_name, objname=object_name):
             return False
 
+        if flag:
+            found_flag = _get_flag(flag)
+            if not bool(found_flag):
+                return False
+            flag_tuple = (found_flag,)
+
         if args is not None:
             classobj = None
 
@@ -275,14 +349,16 @@ def _check_support(conn, feature, obj=None):
             # is of the correct type
             if object_name:
                 classobj = _get_command(object_name)
-                if not isinstance(obj, classobj):
+                if not isinstance(data, classobj):
                     raise ValueError("Passed obj with args must be of type " +
                                      str(classobj))
 
-            cmd = _get_command(function_name, obj=obj)
+            cmd = _get_command(function_name, obj=data)
 
             # Function with args specified is all the proof we need
-            return _try_command(cmd, args)
+            ret = _try_command(cmd, args + flag_tuple,
+                               check_all_error=bool(flag_tuple))
+            return ret
 
     # Check that local libvirt version is sufficient
     if minimum_libvirt_version > actual_lib_ver:
@@ -294,17 +370,44 @@ def _check_support(conn, feature, obj=None):
 
     # If driver specific version info specified, try to verify
     if drv_version:
+        found = False
         for drv, min_drv_ver in drv_version:
             if drv != drv_type:
                 continue
 
-            if min_drv_ver >= actual_drv_ver:
-                return True
+            if min_drv_ver < 0:
+                if actual_drv_ver <= -min_drv_ver:
+                    found = True
+                    break
+            else:
+                if actual_drv_ver >= min_drv_ver:
+                    found = True
+                    break
 
-    else:
-        return True
+        if not found:
+            return False
 
-    return False
+    if hv_version:
+        found = False
+        hv_type = data
+        for hv, min_hv_ver in hv_version:
+            if hv != hv_type:
+                continue
+
+            # XXX: No HV specific version info, just use driver version
+            if min_hv_ver < 0:
+                if actual_drv_ver <= -min_hv_ver:
+                    found = True
+                    break
+            else:
+                if actual_drv_ver >= min_hv_ver:
+                    found = True
+                    break
+
+        if not found:
+            return False
+
+    return True
 
 # Public API below
 
@@ -330,10 +433,12 @@ def support_threading():
 def support_openauth():
     return bool(_local_lib_ver() >= 4000)
 
-
 def check_conn_support(conn, feature):
     return _check_support(conn, feature, conn)
 
+def check_conn_hv_support(conn, feature, hv):
+    return _check_support(conn, feature, hv)
+
 def check_domain_support(dom, feature):
     return _check_support(_get_conn_from_object(dom), feature, dom)
 
@@ -342,3 +447,6 @@ def check_pool_support(pool, feature):
 
 def check_nodedev_support(nodedev, feature):
     return _check_support(_get_conn_from_object(nodedev), feature, nodedev)
+
+def check_interface_support(nodedev, feature):
+    return _check_support(_get_conn_from_object(nodedev), feature, nodedev)
